<?php
/**
 * Joomla! 1.5 component Collector
 *
 * @version 	$Id: field.php 134 2012-04-05 20:45:38Z steevo $
 * @author 		Philippe Ousset steevo@steevo.fr
 * @package 	Collector
 * @license 	GNU/GPL
 *
 * Collector is a Multi Purpose Listing Tool.
 * Originaly developped to list Collections
 * it can be used for several purpose.
 */

// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die( 'Restricted access' );

jimport('joomla.form.formfield');
jimport('joomla.database.database');
jimport('joomla.database.table');

/**
 * Field abstract class
 *
 * @package	Collector
 * @abstract
 */
class CollectorField
{
	/**
	 * type
	 * 
	 * @var string
	 */
	public $type = null;
	
	/**
	 * Collection Id
	 * 
	 * @var int
	 */
	var $_collection;
	
	/**
	 * TableCollector_fields object
	 * 
	 * @var object TableCollector_fields
	 */
	var $_field;
	
	/**
	 * Item Id
	 * 
	 * @var int
	 */
	var $_item;
	
	/**
	 * Value filtered in menu parameters
	 * 
	 * @var mixed
	 */
	var $valueFilterMenu = null;
	
	/**
	 * Object constructor to set field
	 *
	 * Can be overloaded/supplemented by the child class
	 *
	 * @access	protected
	 * @param	int								$collection	Collection Id
	 * @param	object TableCollector_fields	$field		TableCollector_fields object
	 * @param	int								$item		Item Id
	 */
	function __construct( $collection, $field, $item = 0 )
	{
		// Initialisation
		$this->_collection = $collection;
		$this->_item = $item;
		if (is_object ($field))
		{
			$this->_field = $field;
		}
		else
		{
			$this->_field = JTable::getInstance('Collector_fields_type','Table');
			$this->_field->load($field);
		}
	}
	
	/**
	 * Returns a reference to the a user collection, always creating it
	 *
	 * @param	int								$collection	Collection Id
	 * @param	object TableCollector_fields	$field		TableCollector_fields object
	 * @param	int								$item		Item Id
	 * @return	object CollectorField						Reference to a field class
	 */
	function &getInstance( $collection, $field, $item = 0 )
	{
		if (is_object ($field))
		{
			$this->_field = $field;
		}
		else
		{
			$this->_field = JTable::getInstance('Collector_fields_type','Table');
			$this->_field->load($field);
		}
		$fieldClass = 'CollectorField_'.ucfirst($this->_field->type);

		if (!class_exists( $fieldClass ))
		{
			jimport('joomla.filesystem.file');
			$path = JPATH_ROOT.DS.'administrator'.DS.'components'.DS.'com_collector'.DS.'classes'.DS.'field'.DS.strtolower($this->_field->type).DS.'field.'.strtolower($this->_field->type).'.php';
			if( JFile::exists($path) )
			{
				require_once $path;

				if (!class_exists( $fieldClass ))
				{
					return JError::raiseError( 404, 'Field class ' . $fieldClass . ' not found in file.' );
				}
			}
			else
			{
				return JError::raiseError( 404, 'Field type ' . $this->_field->type . ' not supported. File '.$path.' not found.' );
			}
		}

		$instance = new $fieldClass($collection,$this->_field,$item);

		// load language
		$extension = 'com_collector';		
		$basePath = JPATH_ROOT.DS.'administrator'.DS.'components'.DS.'com_collector'.DS.'classes'.DS.'field'.DS.strtolower($this->_field->type);
		
		$language = JFactory::getLanguage();
		$language->load($extension, $basePath);
		
		return $instance;
	}
	
	/**
	 * Gets the field attributes for the form definition
	 *
	 * @return array
	 */
	function getFieldAttributes( $attributes = array() )
	{
		$standardAttributes = array(
			'name'			=> $this->_field->tablecolumn,
			'type'			=> "Collector".ucfirst($this->type),
			'label'			=> $this->_field->field,
			'description'	=> $this->_field->description
		);
		if ( $this->_field->required == 1 ) {
			$required = array('required' => "true");
			$standardAttributes = array_merge($standardAttributes, $required);
		}
		
		$result = array_merge($standardAttributes, $attributes);
		return $result;
	}
	
	/**
	 * Gets the action to do on submit
	 *
	 * @return string
	 */
	function onSubmit(&$form)
	{
		return;
	}
	
	/**
	 * Gets the action to do on submit
	 *
	 * @return string
	 */
	function onRegisterField(&$form)
	{
		return;
	}
	
	/**
	 * Gets the type of field
	 *
	 * @return string
	 */
	function getFieldType()
	{
		return $this->_field->type;
	}
	
	/**
	 * Gets the name of field
	 *
	 * @return string
	 */
	function getFieldName()
	{
		return $this->_field->field;
	}
	
	/**
	 * Method to add field to query
	 *
	 * Can be overloaded/supplemented by the child class
	 *
	 * @param	JDatabaseQuery object		$query
	 */
	function setQuery(&$query)
	{
		$query->select('h.'.$this->_field->tablecolumn);
		return;
	}
	
	/**
	 * Method to add where clause to query on search value
	 *
	 * Can be overloaded/supplemented by the child class
	 *
	 * @param	JDatabaseQuery object		$query
	 * @param	string						$search_all_value
	 */
	function getSearchWhereClause(&$query,$search_all_value)
	{
		$where = 'h.'.$this->_field->tablecolumn.' LIKE "%'.$search_all_value.'%"';
		return $where;
	}
	
	/**
	 * Method to add where clause to query on filter value
	 *
	 * Can be overloaded/supplemented by the child class
	 *
	 * @param	JDatabaseQuery object		$query
	 * @param	mixed						$value
	 */
	function setFilterWhereClause(&$query,$value)
	{
		return false;
	}
	
	/**
	 * Method to add order by clause
	 *
	 * Can be overloaded/supplemented by the child class
	 */
	function getOrderBy()
	{
		$orderBy = 'h.'.$this->_field->tablecolumn;
		return $orderBy;
	}
	
	/**
	 * Method to get specific attributes
	 *
	 * @param	SimpleXMLElement object		$formXML
	 */
	function getFieldsetAttribs(&$formXML)
	{
		$path = JPath::find(JPATH_ROOT.DS.'administrator'.DS.'components'.DS.'com_collector'.DS.'classes'.DS.'field'.DS.strtolower($this->type), 'field.xml');
		$fieldXML = JFactory::getXML($path);
		
		$formFieldset = $formXML->xpath('/form');
		
		$attributes = array(
			'name'			=> 'attribs-'.strtolower($this->type)
		);
		$child = $formFieldset[0]->addChild('fields');
		foreach ($attributes as $key => $value)
		{
			$child->addAttribute($key,$value);
		}
		
		$fieldset = $fieldXML->xpath('/install/config/fields');
		
		$this->_insertXML($fieldset[0],$child);
		
		return;
	}
	
	/**
	 * Method to copy xml
	 *
	 * Copy all children from source to destination
	 *
	 * @param	SimpleXMLElement object		$source
	 * @param	SimpleXMLElement object		$destination
	 */
	private function _insertXML(&$source,&$destination)
	{
		$childs = $source->children();
		
		foreach ( $childs as $child )
		{
			$name = $child->getName();
			$attributes = $child->attributes();
			$value = '';
			if ($name == 'option')
			{
				$option = $child->asXML();
				$match = array();
				$pattern  = "#<option.*>([A-Z_]+)</option>#";
				if(preg_match($pattern, $option, $match)) {
					$value = $match[1];
				}
			}
			
			$newfield = $destination->addChild($name,$value);
			foreach ($attributes as $key => $value)
			{
				$newfield->addAttribute($key,$value);
			}
			
			if ( $child->count() > 0 )
			{
				$this->_insertXML($child,$newfield);
			}
		}
		
		return;
	}
	
	/**
	 * Method to display field
	 *
	 * Can be overloaded/supplemented by the child class
	 *
	 * @param	string		$value		Field value
	 * @param	boolean		$listing	Field value
	 */
	function display($value,$listing=true)
	{
		echo $value;
		return;
	}
	
	/**
	 * Method to display field for edition
	 *
	 * Can be overloaded/supplemented by the child class
	 *
	 * @param	string		$value	Field value
	 */
	function displayEdition($value)
	{
		echo $value;
		return;
	}
	
	/**
	 * Method to display filter in search area
	 *
	 * Can be overloaded/supplemented by the child class
	 *
	 * @param	JRegistry object		$params
	 */
	function displayFilter($params)
	{
		return;
	}
	
	/**
	 * Method to display filter in search area
	 *
	 * Can be overloaded/supplemented by the child class
	 *
	 * @param	JRegistry object		$params
	 */
	function displayRequired($params,$value)
	{
		$html[] = '<div style="display: block; margin-right: 10px; margin-bottom: 10px" >';
		$html[] = '<div style="display: block;" >' . $this->_field->field.'</div>';
		$html[] = '<div style="display: block; clear: none;" >';
		$checked = $value?'checked ':'';
		$html[] = '<input type="radio" id="jform_params_requiredfilterfield_' . $this->_field->tablecolumn . '0" name="jform[params][required][filterfield_' .$this->_field->tablecolumn. ']"' . ' value="1" style="display: inline; float: left;" '.$checked.'/>';
		$html[] = '<label for="jform_params_requiredfilterfield_' . $this->_field->tablecolumn . '" style="display: inline; clear: none; min-width: 60px" >' . JText::_('JYES') . '</label>';
		$checked = $value?'':'checked ';
		$html[] = '<input type="radio" id="jform_params_requiredfilterfield_' . $this->_field->tablecolumn . '1" name="jform[params][required][filterfield_' .$this->_field->tablecolumn. ']"' . ' value="0" style="display: inline; clear: none;" '.$checked.'/>';
		$html[] = '<label for="jform_params_requiredfilterfield_' . $this->_field->tablecolumn . '" style="display: inline; clear: none; min-width: 60px" >' . JText::_('JNO') . '</label>';
		$html[] = '</div>';
		$html[] = '</div>';
		
		return implode($html);
	}
	
	/**
	 * Method to know if field is filtered in menu parameters
	 *
	 * Can be overloaded/supplemented by the child class
	 *
	 * @param	JRegistry object		$params
	 */
	function isFiltered($params)
	{
		$filter = $params->get('filter');
		if (is_array($filter)) {
			$filter = JArrayHelper::toObject($filter);
		}
		
		$nameFilterCollection = 'filterfield_'.$this->_field->tablecolumn;
		if (isset($filter->$nameFilterCollection)) {
			$this->valueFilterMenu = $filter->$nameFilterCollection;
		} else {
			$this->valueFilterMenu = 0;
		}
		
		if ($this->valueFilterMenu == 0) {
			return false;
		} else {
			return true;
		}
		return false;
	}
}