<?php
/**
 * Joomla! 1.5 component Collector
 *
 * @version 	$Id: field.item.php 131 2012-03-19 23:33:03Z steevo $
 * @author 		Philippe Ousset steevo@steevo.fr
 * @package 	Collector
 * @license 	GNU/GPL
 *
 * Collector is a Multi Purpose Listing Tool.
 * Originaly developped to list Collections
 * it can be used for several purpose.
 */

// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die( 'Restricted access' );

/**
 * Field Item class
 *
 * @package	Collector
 */
class CollectorField_Item extends CollectorField
{
	/**
	 * type
	 * 
	 * @var string
	 */
	public $type = 'item';
	
	/**
	 * Object constructor to set field
	 *
	 * Can be overloaded/supplemented by the child class
	 *
	 * @access	protected
	 * @param	int								$collection	Collection Id
	 * @param	object TableCollector_fields	$field		TableCollector_fields object
	 * @param	int								$item		Item Id
	 */
	function __construct( $collection, $field, $item = 0 )
	{
		// Initialisation
		$this->_collection = $collection;
		$this->_item = $item;
		$this->_field = $field;
	}
	
	/**
	 * Gets the field attributes for the form definition
	 *
	 * @return string
	 */
	function getFieldAttributes()
	{
		$attributes = array(
			'collection'	=> $this->_field->attribs['collection'],
			'default'		=> ''
		);
		
		return parent::getFieldAttributes($attributes);
	}
	
	/**
	 * Method to display field
	 *
	 * Can be overloaded/supplemented by the child class
	 *
	 * @param	string		$value		Field value
	 * @param	boolean		$listing	Field value
	 */
	function display($value,$listing=true)
	{
		$db =& JFactory::getDBO();
		
		// recuperation du titre personnalise pour la collection
		$collection = $this->_field->attribs['collection'];
		
		$query = 'SELECT custom';
		$query .= ' FROM `#__collector`';
		$query .= ' WHERE id = \'' . $collection . '\'';
		
		$db->setQuery($query);
		$row = $db->loadObjectList();
		
		if ( $row[0]->custom == "0" )
		{
			$query = 'SELECT id';
			$query .= ' FROM `#__collector_fields`';
			$query .= ' WHERE collection = \'' . $collection . '\'';
			$query .= ' AND home = \'1\'';
			
			$db->setQuery($query);
			$row = $db->loadObjectList();
			
			$custom[] = $row[0]->id;
		}
		else
		{
			$custom = explode( '/', $row[0]->custom );
		}
		
		$query = 'SELECT id, field, tablecolumn, type, defined';
		$query .= ' FROM `#__collector_fields`';
		$query .= ' WHERE collection = \'' . $collection . '\'';
		
		$db->setQuery($query);
		$results = $db->loadObjectList();
		
		$i = 0;
		
		foreach ( $custom as $field )
		{
			for ($j = 0; $j < count($results); $j ++)
			{
				$number_field_default[$i] = $field;
				
				if ( $results[$j]->id == $field )
				{
					$name_field_default[$i] = $results[$j]->field;
					$column_field_default[$i] = $results[$j]->tablecolumn;
					if ( $results[$j]->type == 3 )
					{
						$defined_field_default[$i] = $results[$j]->defined;
					}
					else
					{
						$defined_field_default[$i] = null;
					}
				}
			}
			$i ++;
		}
		
		// recherche des objets de la collection
		//$query = 'SELECT c.id, c.state, ordering, hits, created, created_by, u.name AS author, created_by_alias, access, g.name AS groupname, publish_up, publish_down, ';
		$query = 'SELECT c.id,';
		for ( $i=0; $i < count($number_field_default); $i ++ )
		{
			if ($defined_field_default[$i])
			{
				$query .= ' d'.$i.'.content AS name'.$i.',';
			}
			else
			{
				$query .= ' h.'.$column_field_default[$i].' AS name'.$i.',';
			}
		}
		$query .= ' CASE WHEN CHAR_LENGTH(c.alias) THEN CONCAT_WS(\':\', c.id, c.alias) ELSE c.id END as slug';
		$query .= ' FROM `#__collector_items` AS c';
		$query .= ' LEFT JOIN `#__users` AS u ON c.created_by = u.id';
		$query .= ' LEFT JOIN `#__groups` AS g ON c.access = g.id';
		$query .= ' LEFT JOIN `#__collector_items_history_' . $collection . '` AS h ON h.item = c.id';
		for ( $i=0; $i < count($number_field_default); $i ++ )
		{
			if ($defined_field_default[$i])
			{
				$query .= ' LEFT JOIN `#__collector_defined_content` AS d'.$i.' ON h.' . $column_field_default[$i] . ' = d'.$i.'.id';
			}
		}
		$query .= ' WHERE h.state = 1';
		$query .= ' AND c.collection = \'' . $collection . '\'';
		$query .= ' AND c.id = \'' . $value . '\'';
		$query .= ' ORDER BY c.ordering';
		
		$db->setQuery($query);
		$row = $db->loadObject();
		
		if ($row == null)
		{
			return false;
		}
		$name = $row->name0;
		
		for ($j=1; $j<count($number_field_default); $j ++)
		{
			$field = 'name'.$j;
			$name .= ' ' . $row->$field;
		}
		
		$row->name = $name;
		
		$link = JRoute::_('index.php?option=com_collector&view=item&id='.$row->collection.'&item='.$row->slug);
		
		echo '<a href="'.$link.'">'.$row->name.'</a>';
		
		return;
	}
}

require_once(JPATH_ROOT.DS.'libraries'.DS.'joomla'.DS.'form'.DS.'fields'.DS.'list.php');

/**
 * Form Field class for the Collector.
 *
 * @package		Collector
 */
class JFormFieldCollectorItem extends JFormFieldList
{
	public $type 		= 'CollectorItem';
	
	protected function getOptions()
	{
		// Get item title
		$collection = $this->element['collection'];
		
		// Create a new query object.
		$db =& JFactory::getDBO();
		$query = $db->getQuery(true);
		
		// Select the custom title for items.
		$query->select('custom');
		$query->from('#__collector');
		$query->where('id = '.$collection);
		$db->setQuery( $query );
		
		$custom = $db->loadResult();
		
		// Create a new query object.
		$query = $db->getQuery(true);
		
		// Select the required fields from the table.
		$query->select('f.id, f.field, f.tablecolumn, f.defined');
		$query->from('#__collector_fields as f');
		
		// Join over the type.
		$query->select('t.name AS type');
		$query->join('LEFT', '#__collector_fields_type AS t ON t.id = f.type');
		
		// Filter by collection
		$query->where('f.collection = '.$collection->id);
		
		// Filter by title
		if ( $custom != '0' )
		{
			$custom = explode('/',$custom);
			$query->where('f.id IN("'.implode('","',$custom).'")');
		}
		else
		{
			$query->where('home = 1');
		}
		
		$db->setQuery( $query );
		
		$fields = $db->loadObjectList();
		
		// Reorder
		$fieldsReordered = $fields;
		if ( $collection->custom != '0' )
		{
			$custom = array_flip($custom);
			foreach( $fields as $field )
			{
				$fieldsReordered[$custom[$field->id]] = $field;
			}
		}
		
		foreach ($fieldsReordered as $field)
		{
			$fieldObjects[] =& CollectorField::getInstance( $collection, $field );
		}
		
		// Create a new query object.
		$query = $db->getQuery(true);
		
		// Select the required fields from the table.
		$query->select( 'i.id' );
		$query->from('#__collector_items AS i');
		
		// Join over the values.
		$query->join('LEFT', '#__collector_items_history_'.$collection->id.' AS h ON h.item = i.id');
		foreach($fieldObjects as $field)
		{
			$field->setQuery($query);
		}
		
		// Filter by collection
		$query->where('i.collection = '.$collection->id);
		
		// Filter by history
		$query->where('h.state = 1');
		
		$db->setQuery( $query );
		
		$results = $db->loadObjectList();
		//Check for database error
		if (!$db->query())
		{
			return JError::raiseWarning( 500, $db->getErrorMsg() );
		}
		
		array_unshift($results,array('value' => 0, 'text' => ''));
		
		return $results;
	}
}