<?php
/**
 * Joomla! 1.5 component Collector
 *
 * @version 	$Id: templates.php 120 2011-09-04 22:33:31Z steevo $
 * @author 		Philippe Ousset steevo@steevo.fr
 * @package 	Collector
 * @license 	GNU/GPL
 *
 * Collector is a Multi Purpose Listing Tool.
 * Originaly developped to list Collections
 * it can be used for several purpose.
 */

// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die( 'Restricted access' );

jimport('joomla.application.component.controller');

/**
 * Templates Controller
 *
 * @package  	Collector
 */
class CollectorsControllerTemplates extends CollectorsController
{
	/**
	 * Method to edit template
	 *
	 * @access	public
	 */
	function edit()
	{
		JRequest::setVar('view', 'templateedit');
		JRequest::setVar('hidemainmenu',1);
		JRequest::setVar('edit', 'edit');
		
		parent::display();
	}
	
	/**
	 * Method to copy template
	 *
	 * @access	public
	 */
	function copy()
	{
		JRequest::setVar('view', 'templateedit');
		JRequest::setVar('hidemainmenu',1);
		JRequest::setVar('edit', 'copy');
		
		parent::display();
	}
	
	/**
	 * Method to edit source template
	 *
	 * @access	public
	 */
	function edit_source()
	{
		JRequest::setVar('view', 'htmledit');
		JRequest::setVar('hidemainmenu',1);
		
		parent::display();
	}
	
	/**
	 * Method to add template
	 *
	 * @access	public
	 */
	function add()
	{
		JRequest::setVar('view', 'templateedit');
		JRequest::setVar('hidemainmenu',1);
		JRequest::setVar('edit', 'add');
		
		parent::display();
	}
	
	/**
	 * Cancels an edit template operation
	 *
	 * @access	public
	 */
	function cancel()
	{
		$option = JRequest::getCmd( 'option' );
		$view = JRequest::getCmd( 'view' );
		$collection = JRequest::getCmd( 'collection' );
		
		$model = $this->getModel('templateedit');
				
		$this->setRedirect( 'index.php?option='.$option.'&view='.$view.'&collection='.$collection, $msg );
	}
	
	/**
	 * Method to save template
	 *
	 * @access	public
	 */
	function save()
	{
		$option = JRequest::getCmd( 'option' );
		$view = JRequest::getCmd( 'view' );
		$collection = JRequest::getCmd( 'collection' );
		
		$model = $this->getModel('templateedit');
		
		if ($model->store())
		{
			$msg = JText::_( 'COM_COLLECTOR_TPL_SAVED' );
		}
		else
		{
			$msg = JText::_( 'COM_COLLECTOR_ERROR_SAVE' );
		}
		
		$this->setRedirect( 'index.php?option='.$option.'&view='.$view.'&collection='.$collection, $msg );
	}
	
	/**
	 * Method to save source template
	 *
	 * @access	public
	 */
	function save_source()
	{
		$option = JRequest::getCmd( 'option' );
		$view = JRequest::getCmd( 'view' );
		$collection = JRequest::getCmd( 'collection' );
		
		$model = $this->getModel('htmledit');
		
		if ($model->store())
		{
			$msg = JText::_( 'COM_COLLECTOR_TPL_SAVED' );
		}
		else
		{
			$msg = JText::_( 'COM_COLLECTOR_ERROR_SAVE' );
		}
		
		$this->setRedirect( 'index.php?option='.$option.'&view='.$view.'&collection='.$collection, $msg );
	}
	
	/**
	 * Method to save template and reload edition
	 *
	 * @access	public
	 */
	function apply()
	{
		$option = JRequest::getCmd( 'option' );
		$id = JRequest::getCmd( 'id' );
		
		$model = $this->getModel('templateedit');
		
		if ($model->store())
		{
			$msg = JText::_( 'COM_COLLECTOR_TPL_SAVED' );
		}
		else
		{
			$msg = JText::_( 'COM_COLLECTOR_ERROR_SAVE' );
		}
		
		$this->setRedirect( 'index.php?option='.$option.'&view=templateedit&action=edit&cid[]='.$id, $msg );
	}
	
	/**
	 * Method to delete template
	 *
	 * @access	public
	 */
	function remove()
	{
		$option = JRequest::getCmd( 'option' );
		$view = JRequest::getCmd( 'view' );
		
		$model = $this->getModel('templates');
		
		$msg = $model->remove();
		
		$this->setRedirect( 'index.php?option='.$option.'&view='.$view, $msg );
	}
	
	/**
	 * Method to set template as default
	 *
	 * @access	public
	 */
	function setdefault()
	{
		// Check for request forgeries
		JRequest::checkToken() or jexit( 'Invalid Token' );
		
		$option = JRequest::getCmd( 'option' );
		$view = JRequest::getCmd( 'view' );
		$collection = JRequest::getCmd( 'collection' );
		
		// Get some variables from the request
		$cid	= JRequest::getVar( 'cid', array(), 'post', 'array' );
		JArrayHelper::toInteger($cid);
		
		if (isset($cid[0]) && $cid[0]) {
			$id = $cid[0];
		} else {
			$this->setRedirect( 'index.php?option='.$option.'&view='.$view.'&collection='.$collection, JText::_('COM_COLLECTOR_NO_ITEMS_SELECTED') );
			return false;
		}
		
		$model =& $this->getModel( 'templates' );
		if ($model->setHome($id)) {
			$msg = JText::_( 'COM_COLLECTOR_DEFAULT_TEMPLATE_SET' );
		} else {
			$msg = $model->getError();
		}
		$this->setRedirect( 'index.php?option='.$option.'&view='.$view.'&collection='.$collection, $msg );
	}
	
	/**
	 * Method to display a view
	 *
	 * @access	public
	 */
	function display()
	{
		parent::display();
	}
}