<?php
/**
 * Joomla! 1.5 component Collector
 *
 * @version 	$Id: collectionfields.php 126 2011-11-09 21:54:29Z steevo $
 * @author 		Philippe Ousset steevo@steevo.fr
 * @package 	Collector
 * @license 	GNU/GPL
 *
 * Collector is a Multi Purpose Listing Tool.
 * Originaly developped to list Collections
 * it can be used for several purpose.
 */

// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die( 'Restricted access' );

jimport( 'joomla.application.component.model' );

if(version_compare(JVERSION,'1.6.0','<')){
//Code pour Joomla! 1.5

/**
 * Collectionfields model
 * @package	Collector
 */
class CollectorsModelCollectionfields extends JModel
{
	/**
	 * Collection Id
	 * @var int
	 */
	var $_collection = null;

	/**
	 * Array of objects containing the fields
	 * @var array
	 */
	var $_data = null;

	/**
	 * Items total
	 * @var int
	 */
	var $_total = null;

	/**
	 * Pagination object
	 * @var object
	 */
	var $_pagination = null;

	/**
	 * Collectionfields model constructor
	 *
	 * @access	public
	 */
	function __construct()
	{
		parent::__construct();
		
		global $mainframe, $option;
		
		$this->_collection = JRequest::getVar( 'collection', '', '', 'int');
		
		$this->_loaddefaults();
		
        // Get pagination request variables// Get folder request variables
		if(version_compare(JVERSION,'1.6.0','<')){
			//Code pour Joomla! 1.5
			$limit = $mainframe->getUserStateFromRequest('global.list.limit', 'limit', $mainframe->getCfg('list_limit'), 'int');
			$limitstart = $mainframe->getUserStateFromRequest($option.'.limitstart', 'limitstart', 0, 'int');
		}else{
			//Code pour Joomla >= 1.6.0
			$app = JFactory::getApplication();
			$limit = $app->getUserStateFromRequest('global.list.limit', 'limit', $app->getCfg('list_limit'), 'int');
			$limitstart = $app->getUserStateFromRequest($option.'.limitstart', 'limitstart', 0, 'int');
		}
		
        // In case limit has been changed, adjust it
        $limitstart = ($limit != 0 ? (floor($limitstart / $limit) * $limit) : 0);
		
        $this->setState('limit', $limit);
        $this->setState('limitstart', $limitstart);
	}
	
	/**
	 * Method to load default collection Id in <var>_collection</var>
	 *
	 * @access	private
	 */
	function _loaddefaults()
	{
		if ($this->_collection == '')
		{	
			$query = 'SELECT id';
			$query .= ' FROM `#__collector`';
			$query .= ' WHERE home = 1';
			
			$row = $this->_getList( $query );
			
			if ( !$row )
			{
				/**
				* tag  defaut d'une collection
				*/
				$query = 'UPDATE `#__collector` SET home = 1 LIMIT 1;';
				$this->_db->setQuery( $query );
				$this->_db->query();
				
				$query = 'SELECT id';
				$query .= ' FROM `#__collector`';
				$query .= ' WHERE home = 1';
				
				$row = $this->_getList( $query );
			}
			$this->_collection = $row[0]->id;
		}
	}
	
	/**
	 * Method to create the query to select fields
	 *
	 * @access	private
	 * @return	string			Query to select fields
	 */
	function _buildQuery()
	{
		// Get the WHERE and ORDER BY clauses for the query
		$where		= $this->_buildContentWhere();
		$orderby	= $this->_buildContentOrderBy();
		
		$query = 'SELECT f.id, f.collection, f.field, f.tablecolumn, f.type, ordering, state, created, created_by, u.name AS author, created_by_alias, modified, modified_by, access, home, edit, unik, listing, filter, sort, g.name AS groupname, publish_up, publish_down';
		$query .= ' FROM `#__collector_fields` AS f';
		$query .= ' LEFT JOIN `#__users` AS u ON f.created_by = u.id';
		$query .= ' LEFT JOIN `#__groups` AS g ON f.access = g.id';
		
		$query .= $where;
		$query .= $orderby;
		
		return $query;
	}
	
	/**
	 * Method to build the ORDER BY clause of the query to select fields
	 *
	 * @access	private
	 * @return	string	ORDER BY clause
	 */
	function _buildContentOrderBy()
	{
		global $mainframe, $option;
		
		// Get folder request variables
		if(version_compare(JVERSION,'1.6.0','<')){
			//Code pour Joomla! 1.5
			$filter_order		= $mainframe->getUserStateFromRequest( $option.'.admin.fields.filter_order',		'filter_order',		'f.ordering',	'cmd' );
			$filter_order_Dir	= $mainframe->getUserStateFromRequest( $option.'.admin.fields.filter_order_Dir',	'filter_order_Dir',	'',				'word' );
		}else{
			//Code pour Joomla >= 1.6.0
			$app = JFactory::getApplication();
			$filter_order		= $app->getUserStateFromRequest( $option.'.admin.fields.filter_order',		'filter_order',		'f.ordering',	'cmd' );
			$filter_order_Dir	= $app->getUserStateFromRequest( $option.'.admin.fields.filter_order_Dir',	'filter_order_Dir',	'',				'word' );
		}
		
		if ($filter_order == 'f.ordering'){
			$orderby 	= ' ORDER BY f.ordering '.$filter_order_Dir;
		} else {
			$orderby 	= ' ORDER BY '.$filter_order.' '.$filter_order_Dir.' , f.ordering ';
		}
		
		return $orderby;
	}
	
	/**
	 * Method to build the WHERE clause of the query to select fields
	 *
	 * @access	private
	 * @return	string	WHERE clause
	 */
	function _buildContentWhere()
	{
		global $mainframe, $option;
		
		$db					=& JFactory::getDBO();
		
		// Get folder request variables
		if(version_compare(JVERSION,'1.6.0','<')){
			//Code pour Joomla! 1.5
			$filter_state		= $mainframe->getUserStateFromRequest( $option.'.admin.fields.filter_state',		'filter_state',		'',				'word' );
			$search				= $mainframe->getUserStateFromRequest( $option.'.admin.fields.search',				'search',			'',				'string' );
		}else{
			//Code pour Joomla >= 1.6.0
			$app = JFactory::getApplication();
			$filter_state		= $app->getUserStateFromRequest( $option.'.admin.fields.filter_state',		'filter_state',		'',				'word' );
			$search				= $app->getUserStateFromRequest( $option.'.admin.fields.search',				'search',			'',				'string' );
		}
		$search				= JString::strtolower( $search );
		
		$where = array();
		
		$where[] = 'f.collection = \'' . $this->_collection . '\'';
		
		if ($search) {
			$where[] = 'LOWER(f.field) LIKE '.$db->Quote( '%'.$db->getEscaped( $search, true ).'%', false );
		}
		if ( $filter_state ) {
			if ( $filter_state == 'P' ) {
				$where[] = 'f.state >= 1';
			} else if ($filter_state == 'U' ) {
				$where[] = 'f.state = 0';
			}
		}
		
		$where 		= ( count( $where ) ? ' WHERE '. implode( ' AND ', $where ) : '' );
		
		return $where;
	}
	
	/**
	 * Retrieves the fields
	 *
	 * @access	public
	 * @return	array	Array of objects containing the fields from the database
	 */
	function &getData()
	{
		// Lets load the fields if it doesn't already exist
		if ( empty( $this->_data ))
		{
			$query = $this->_buildQuery();
			$this->_data = $this->_getList( $query, $this->getState('limitstart'), $this->getState('limit') );
		}
		
		return $this->_data;
	}
	
	/**
	 * Retrieves the collection Id
	 *
	 * @access	public
	 * @return	int		Collection Id
	 */
	function getCollection()
	{
		return $this->_collection;
	}
	
	/**
	 * Method to get number of results
	 * Put data in <var>_total</var>
	 *
	 * @access	public
	 * @return	int		Number of results
	 */
	function getTotal()
	{
		// Load the content if it doesn't already exist
		if (empty($this->_total))
		{
			$query = $this->_buildQuery();
			$this->_total = $this->_getListCount($query);    
		}
		return $this->_total;
	}

	/**
	 * Method to get pagination
	 * Put data in <var>_pagination</var>
	 * Returns a reference to the global {@link JPagination} object
	 *
	 * @access	public
	 * @return	object JPagination
	 */
	function getPagination()
	{
		// Load the content if it doesn't already exist
		if (empty($this->_pagination))
		{
			jimport('joomla.html.pagination');
			$this->_pagination = new JPagination($this->getTotal(), $this->getState('limitstart'), $this->getState('limit') );
		}
		return $this->_pagination;
	}

	/**
	 * Method to delete fields
	 *
	 * @access	public
	 * @return	string	Message to display
	 */
	function remove()
	{
		$cid = JRequest::getVar( 'cid', array(0), 'post', 'array');
		$row =& $this->getTable('collector_fields');
		
		foreach ($cid as $id)
		{
			if (!($row->delete($id)))
			{
				JError::raiseError( 500, $row->getError() );
				return false;
			}
		}
		
		$row->reorder('collection='.$this->_collection);
		
		$msg = count($cid) . ' ' . JText::_( 'COM_COLLECTOR_FIELD_DELETED' );
		
		return $msg;
	}
	
	/**
	 * Method to set frontend edition parameter
	 *
	 * @access	public
	 * @param	int		$state	Parameter state
	 * @return	string			Message to display
	 */
	function edit($state)
	{
		// on initialise les variables
		$cid = JRequest::getVar('cid', array(0), 'post', 'array');
		$row =& $this->getTable('collector_fields');
		
		foreach ( $cid as $id )
		{
			$row->load($id);
			$row->edit = $state;
			
			// on s'assure que l'objet est valide
			if (!$row->check())
			{
				JError::raiseError( 500, $row->getError() );
				return false;
			}
			
			// on enregistre le tout
			if (!$row->store())
			{
				JError::raiseError( 500, $row->getError() );
				return false;
			}
		}
		
		// cration du message de retour
		$msg ='';
		
		if ( $state == 1 )
		{
			$msg = 'COM_COLLECTOR_FIELD_UNLOCKED';
		}
		else if ( $state == 0 )
		{
			$msg = 'COM_COLLECTOR_FIELD_LOCKED';
		}
		
		$msg = count($cid) . ' ' . JText::_( $msg );
		
		return $msg;
	}
	
	/**
	 * Method to set listing view parameter
	 *
	 * @access	public
	 * @param	int		$state	Parameter state
	 * @return	string			Message to display
	 */
	function listing($state)
	{
		// on initialise les variables
		$cid = JRequest::getVar('cid', array(0), 'post', 'array');
		$row =& $this->getTable('collector_fields');
		
		foreach ( $cid as $id )
		{
			$row->load($id);
			$row->listing = $state;
			
			// on s'assure que l'objet est valide
			if (!$row->check())
			{
				JError::raiseError( 500, $row->getError() );
				return false;
			}
			
			// on enregistre le tout
			if (!$row->store())
			{
				JError::raiseError( 500, $row->getError() );
				return false;
			}
		}
		
		// cration du message de retour
		$msg ='';
		
		if ( $state == 1 )
		{
			$msg = 'COM_COLLECTOR_FIELD_DISPLAYED';
		}
		else if ( $state == 0 )
		{
			$msg = 'COM_COLLECTOR_FIELD_HIDE';
		}
		
		$msg = count($cid) . ' ' . JText::_( $msg );
		
		return $msg;
	}
	
	/**
	 * Method to set sorting parameter
	 *
	 * @access	public
	 * @param	int		$state	Parameter state
	 * @return	string			Message to display
	 */
	function sort($state)
	{
		// on initialise les variables
		$cid = JRequest::getVar('cid', array(0), 'post', 'array');
		$row =& $this->getTable('collector_fields');
		
		foreach ( $cid as $id )
		{
			$row->load($id);
			$row->sort = $state;
			
			// on s'assure que l'objet est valide
			if (!$row->check())
			{
				JError::raiseError( 500, $row->getError() );
				return false;
			}
			
			// on enregistre le tout
			if (!$row->store())
			{
				JError::raiseError( 500, $row->getError() );
				return false;
			}
		}
		
		// cration du message de retour
		$msg ='';
		
		if ( $state == 1 )
		{
			$msg = 'COM_COLLECTOR_SORT_ENABLED';
		}
		else if ( $state == 0 )
		{
			$msg = 'COM_COLLECTOR_SORT_DISABLED';
		}
		
		$msg = count($cid) . ' ' . JText::_( $msg );
		
		return $msg;
	}
	
	/**
	 * Method to set unicity parameter
	 *
	 * @access	public
	 * @param	int		$state	Parameter state
	 * @return	string			Message to display
	 */
	function unik($state)
	{
		// on initialise les variables
		$cid = JRequest::getVar('cid', array(0), 'post', 'array');
		$row =& $this->getTable('collector_fields');
		
		foreach ( $cid as $id )
		{
			$row->load($id);
			$row->unik = $state;
			
			// on s'assure que l'objet est valide
			if (!$row->check())
			{
				JError::raiseError( 500, $row->getError() );
				return false;
			}
			
			// on enregistre le tout
			if (!$row->store())
			{
				JError::raiseError( 500, $row->getError() );
				return false;
			}
		}
		
		// cration du message de retour
		$msg ='';
		
		if ( $state == 1 )
		{
			$msg = 'COM_COLLECTOR_FIELD_UNIK';
		}
		else if ( $state == 0 )
		{
			$msg = 'COM_COLLECTOR_FIELD_NOUNIK';
		}
		
		$msg = count($cid) . ' ' . JText::_( $msg );
		
		return $msg;
	}
	
	/**
	 * Method to set filter parameter
	 *
	 * @access	public
	 * @param	int		$state	Parameter state
	 * @return	string			Message to display
	 */
	function filter($state)
	{
		// on initialise les variables
		$cid = JRequest::getVar('cid', array(0), 'post', 'array');
		$row =& $this->getTable('collector_fields');
		
		foreach ( $cid as $id )
		{
			$row->load($id);
			$row->filter = $state;
			
			// on s'assure que l'objet est valide
			if (!$row->check())
			{
				JError::raiseError( 500, $row->getError() );
				return false;
			}
			
			// on enregistre le tout
			if (!$row->store())
			{
				JError::raiseError( 500, $row->getError() );
				return false;
			}
		}
		
		// cration du message de retour
		$msg ='';
		
		if ( $state == 1 )
		{
			$msg = 'COM_COLLECTOR_FILTER_ENABLED';
		}
		else if ( $state == 0 )
		{
			$msg = 'COM_COLLECTOR_FILTER_DISABLED';
		}
		
		$msg = count($cid) . ' ' . JText::_( $msg );
		
		return $msg;
	}
	
	/**
	 * Method to set state parameter
	 *
	 * @access	public
	 * @param	int		$state	Parameter state
	 * @return	string			Message to display
	 */
	function state($state)
	{
		// on initialise les variables
		$cid = JRequest::getVar('cid', array(0), 'post', 'array');
		$row =& $this->getTable('collector_fields');
		$task = JRequest::getCmd('task');
		
		foreach ( $cid as $id )
		{
			$row->load($id);
			$row->state = $state;
			
			// on s'assure que l'objet est valide
			if (!$row->check())
			{
				JError::raiseError( 500, $row->getError() );
				return false;
			}
			
			// on enregistre le tout
			if (!$row->store())
			{
				JError::raiseError( 500, $row->getError() );
				return false;
			}
		}
		
		// cration du message de retour
		$msg ='';
		
		if ( ( $state == 0 ) && ( $task == 'unpublish' ) )
		{
			$msg = 'COM_COLLECTOR_FIELD_UNPUBLISHED';
		}
		else if ( ( $state == 0 ) && ( $task == 'unarchive' ) )
		{
			$msg = 'COM_COLLECTOR_FIELD_UNARCHIVED';
		}
		else if ( $state == 1 )
		{
			$msg = 'COM_COLLECTOR_FIELD_PUBLISHED';
		}
		else if ( $state == -1 )
		{
			$msg = 'COM_COLLECTOR_FIELD_ARCHIVED';
		}
		
		$msg = count($cid) . ' ' . JText::_( $msg );
		
		return $msg;
	}
	
	/**
	 * Method to set access parameter
	 *
	 * @access	public
	 * @param	int		$state	Parameter state
	 * @return	string			Message to display
	 */
	function access($access)
	{
		// on initialise les variables
		$cid = JRequest::getVar('cid', array(0), 'post', 'array');
		$cid = $cid[0];
		
		// on cre et charge l'objet table de la collection
		$row =& $this->getTable('collector_fields');
		$row->load($cid);
		$row->access = $access;
		
		// on s'assure que l'objet est valide
		if (!$row->check())
		{
			JError::raiseError( 500, $row->getError() );
			return false;
		}
		
		// on enregistre le tout
		if (!$row->store())
		{
			JError::raiseError( 500, $row->getError() );
			return false;
		}
	}
	
	/**
	 * Method to save order
	 *
	 * @access	public
	 * @return	string			Message to display
	 */
	function saveOrder()
	{
		// Initialize variables
		$db			= & JFactory::getDBO();
		
		$cid		= JRequest::getVar( 'cid', array(0), 'post', 'array' );
		$order		= JRequest::getVar( 'order', array (0), 'post', 'array' );
		$total		= count($cid);
		$conditions	= array ();
		
		JArrayHelper::toInteger($cid, array(0));
		JArrayHelper::toInteger($order, array(0));
		
		// Instantiate an article table object
		$row =& $this->getTable('collector_fields');
		
		// Update the ordering for items in the cid array
		for ($i = 0; $i < $total; $i ++)
		{
			$row->load( (int) $cid[$i] );
			if ($row->ordering != $order[$i]) {
				$row->ordering = $order[$i];
				if (!$row->store()) {
					JError::raiseError( 500, $db->getErrorMsg() );
					return false;
				}
				// remember to updateOrder this group
				$condition = 'collection=' . (int) $row->collection . ' AND state >= 0';
				$found = false;
				foreach ($conditions as $cond)
					if ($cond[1] == $condition) {
						$found = true;
						break;
					}
				if (!$found)
					$conditions[] = array ($row->id, $condition);
			}
		}
		
		// execute updateOrder for each group
		foreach ($conditions as $cond)
		{
			$row->load($cond[0]);
			$row->reorder($cond[1]);
		}
		
		$cache = & JFactory::getCache('com_collector');
		$cache->clean();
		
		$msg = JText::_('COM_COLLECTOR_NEW_ORDERING_SAVED');
		
		return $msg;
	}
	
	/**
	 * Method to move the order of a record
	 *
	 * @access	public
	 * @param	int		$direction	The increment to reorder by
	 * @return	string				Message to display
	 */
	function orderField($direction)
	{
		$db		= & JFactory::getDBO();
		
		// Initialize variables
		$cid	= JRequest::getVar( 'cid', array(), 'post', 'array' );
		
		if (isset( $cid[0] ))
		{
			$row =& $this->getTable('collector_fields');
			if (!$row->load( (int) $cid[0] ))
			{
				$this->setError($db->getErrorMsg());
				return false;
			}
			
			if (!$row->move($direction, 'collection = ' . (int) $row->collection . ' AND state >= 0' ))
			{
				$this->setError($db->getErrorMsg());
				return false;
			}
			
			$cache = & JFactory::getCache('com_collector');
			$cache->clean();
		}
		
		return true;
	}
	
	/**
	 * Method to set a field as default
	 *
	 * @access	public
	 * @param	int		$item	Field Id
	 * @return	boolean			True on success
	 */
	function setHome( $item )
	{
		$db =& $this->getDBO();
		
		// Clear home field for all other items
		$query = 'UPDATE #__collector_fields' .
				' SET home = 0' .
				' WHERE 1' .
				' AND collection = ' . $this->_collection;
		$db->setQuery( $query );
		if ( !$db->query() ) {
			$this->setError($db->getErrorMsg());
			return false;
		}
		
		// Set the given item to home
		$query = 'UPDATE #__collector_fields' .
				' SET home = 1' .
				' WHERE id = '.(int) $item;
		$db->setQuery( $query );
		if ( !$db->query() ) {
			$this->setError($db->getErrorMsg());
			return false;
		}
		
		return true;
	}
}

}else{
//Code pour Joomla >= 1.6.0
/**
 * Collectionfields model
 * @package	Collector
 */
class CollectorModelCollectionfields extends JModel
{
	
}

}
