<?php
/**
 * Joomla! 1.5 component Collector
 *
 * @version 	$Id: collections.php 124 2011-09-18 21:51:13Z steevo $
 * @author 		Philippe Ousset steevo@steevo.fr
 * @package 	Collector
 * @license 	GNU/GPL
 *
 * Collector is a Multi Purpose Listing Tool.
 * Originaly developped to list Collections
 * it can be used for several purpose.
 */

// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die( 'Restricted access' );

if(version_compare(JVERSION,'1.6.0','<')){
//Code pour Joomla! 1.5
jimport( 'joomla.application.component.model' );


/**
 * Collections model
 * @package	Collector
 */
class CollectorsModelCollections extends JModel
{
	/**
	 * Array of objects containing the collections
	 * @var array
	 */
	var $_data;

	/**
	 * Collections total
	 * @var int
	 */
	var $_total = null;

	/**
	 * Pagination object
	 * @var object
	 */
	var $_pagination = null;

	/**
	 * Collections model constructor
	 *
	 * @access	public
	 */
	function __construct()
	{
		parent::__construct();
		
		//$array = JRequest::getVar('cid', 0, '', 'array');
		//$this->_id = (int)$array[0];
		
		global $mainframe, $option;
		
        // Get pagination request variables
		// Get folder request variables
		if(version_compare(JVERSION,'1.6.0','<')){
			//Code pour Joomla! 1.5
			$limit = $mainframe->getUserStateFromRequest('global.list.limit', 'limit', $mainframe->getCfg('list_limit'), 'int');
			$limitstart = $mainframe->getUserStateFromRequest($option.'.limitstart', 'limitstart', 0, 'int');
		}else{
			//Code pour Joomla >= 1.6.0
			$app = JFactory::getApplication();
			$limit = $app->getUserStateFromRequest('global.list.limit', 'limit', $app->getCfg('list_limit'), 'int');
			$limitstart = $app->getUserStateFromRequest($option.'.limitstart', 'limitstart', 0, 'int');
		}
		
        // In case limit has been changed, adjust it
        $limitstart = ($limit != 0 ? (floor($limitstart / $limit) * $limit) : 0);
		
        $this->setState('limit', $limit);
        $this->setState('limitstart', $limitstart);
	}
	
	/**
	 * Method to create the query to select collections
	 *
	 * @access	private
	 * @return	string			Query to select collections
	 */
	function _buildQuery()
	{
		$query = 'SELECT c.id, c.name, state, created, created_by, u.name AS author, created_by_alias, modified, modified_by, access, g.name AS groupname, publish_up, publish_down, home FROM `#__collector` AS c';
		$query .= ' LEFT JOIN `#__users` AS u ON c.created_by = u.id';
		$query .= ' LEFT JOIN `#__groups` AS g ON c.access = g.id';
		
		return $query;
	}
	
	/**
	 * Retrieves the collections
	 *
	 * @access	public
	 * @return	array	Array of objects containing the collections from the database
	 */
	function &getData()
	{
		// Lets load the data if it doesn't already exist
		if ( empty( $this->_data ))
		{
			$query = $this->_buildQuery();
			$this->_data = $this->_getList( $query, $this->getState('limitstart'), $this->getState('limit') );
		}
		
		return $this->_data;
	}
	
	/**
	 * Method to get number of results
	 * Put data in <var>_total</var>
	 *
	 * @access	public
	 * @return	int		Number of results
	 */
	function getTotal()
	{
		// Load the content if it doesn't already exist
		if (empty($this->_total))
		{
			$query = $this->_buildQuery();
			$this->_total = $this->_getListCount($query);    
		}
		return $this->_total;
	}

	/**
	 * Method to get pagination
	 * Put data in <var>_pagination</var>
	 * Returns a reference to the global {@link JPagination} object
	 *
	 * @access	public
	 * @return	object JPagination
	 */
	function getPagination()
	{
		// Load the content if it doesn't already exist
		if (empty($this->_pagination))
		{
			jimport('joomla.html.pagination');
			$this->_pagination = new JPagination($this->getTotal(), $this->getState('limitstart'), $this->getState('limit') );
		}
		return $this->_pagination;
	}
	
	/**
	 * Method to delete collections
	 *
	 * @access	public
	 * @return	string	Message to display
	 */
	function remove()
	{
		$cid = JRequest::getVar( 'cid', array(0), 'post', 'array');
		$row =& $this->getTable('collector');
		
		foreach ($cid as $id)
		{
			$row->load($id);
			$home = $row->home;
			
			if (!($row->delete($id)))
			{
				JError::raiseError( 500, $row->getError() );
				return false;
			}
			else
			{
				/**
				* suppression des templates dfinis pour la collection
				*/
				$query = 'DELETE FROM `#__collector_templates` WHERE `collection` = ' . $id . ';';
				
				$this->_db->setQuery( $query );
				$this->_db->query();
				
				/**
				* suppression des champs dfinis pour la collection
				*/
				$query = 'DELETE FROM `#__collector_fields` WHERE `collection` = ' . $id . ';';
				
				$this->_db->setQuery( $query );
				$this->_db->query();
				
				/**
				* suppression des historiques des objets de la collection
				*/
				$query = 'DROP TABLE `#__collector_items_' . $id . '`;';
				
				$this->_db->setQuery( $query );
				$this->_db->query();
				
				/**
				* suppression des objets de la collection
				*/
				$query = 'DROP TABLE `#__collector_' . $id . '`;';
				
				$this->_db->setQuery( $query );
				$this->_db->query();
			}
		}
		
		$msg = count($cid) . ' ' . JText::_( 'COM_COLLECTOR_COLLECTION_N_DELETED' );
		
		return $msg;
	}
	
	/**
	 * Method to set state parameter
	 *
	 * @access	public
	 * @param	int		$state	Parameter state
	 * @return	string			Message to display
	 */
	function state($state)
	{
		// on initialise les variables
		$cid = JRequest::getVar('cid', array(0), 'post', 'array');
		$row =& $this->getTable('collector');
		$task = JRequest::getCmd('task');
		
		foreach ( $cid as $id )
		{
			$row->load($id);
			$row->state = $state;
			
			// on s'assure que l'objet est valide
			if (!$row->check())
			{
				JError::raiseError( 500, $row->getError() );
				return false;
			}
			
			// on enregistre le tout
			if (!$row->store())
			{
				JError::raiseError( 500, $row->getError() );
				return false;
			}
		}
		
		// cration du message de retour
		$msg ='';
		
		if ( ( $state == 0 ) && ( $task == 'unpublish' ) )
		{
			$msg = 'COM_COLLECTOR_COLLECTION_N_UNPUBLISHED';
		}
		else if ( ( $state == 0 ) && ( $task == 'unarchive' ) )
		{
			$msg = 'COM_COLLECTOR_COLLECTION_N_UNARCHIVED';
		}
		else if ( $state == 1 )
		{
			$msg = 'COM_COLLECTOR_COLLECTION_N_PUBLISHED';
		}
		else if ( $state == -1 )
		{
			$msg = 'COM_COLLECTOR_COLLECTION_N_ARCHIVED';
		}
		
		$msg = count($cid) . ' ' . JText::_( $msg );
		
		return $msg;
	}
	
	/**
	 * Method to set access parameter
	 *
	 * @access	public
	 * @param	int		$state	Parameter state
	 * @return	string			Message to display
	 */
	function access($access)
	{
		// on initialise les variables
		$cid = JRequest::getVar('cid', array(0), 'post', 'array');
		$cid = $cid[0];
		
		// on cre et charge l'objet table de la collection
		$row =& $this->getTable('collector');
		$row->load($cid);
		$row->access = $access;
		
		// on s'assure que l'objet est valide
		if (!$row->check())
		{
			JError::raiseError( 500, $row->getError() );
			return false;
		}
		
		// on enregistre le tout
		if (!$row->store())
		{
			JError::raiseError( 500, $row->getError() );
			return false;
		}
	}
	
	/**
	 * Method to set a collection as default
	 *
	 * @access	public
	 * @param	int		$item	Collection Id
	 * @return	boolean			True on success
	 */
	function setHome( $item )
	{
		$db =& $this->getDBO();
		
		// Clear home field for all other items
		$query = 'UPDATE #__collector' .
				' SET home = 0' .
				' WHERE 1';
		$db->setQuery( $query );
		if ( !$db->query() ) {
			$this->setError($db->getErrorMsg());
			return false;
		}
		
		// Set the given item to home
		$query = 'UPDATE #__collector' .
				' SET home = 1' .
				' WHERE id = '.(int) $item;
		$db->setQuery( $query );
		if ( !$db->query() ) {
			$this->setError($db->getErrorMsg());
			return false;
		}
		
		return true;
	}
}

}else{
//Code pour Joomla >= 1.6.0

jimport('joomla.application.component.modellist');

/**
 * Collections model
 * @package	Collector
 */
class CollectorModelCollections extends JModelList
{
	/**
	 * Constructor.
	 *
	 * @param	array	An optional associative array of configuration settings.
	 * @see		JController
	 * @since	1.6
	 */
	public function __construct($config = array())
	{
		if (empty($config['filter_fields'])) {
			$config['filter_fields'] = array(
				'id', 'c.id',
				'name', 'c.title',
				'alias', 'c.alias',
				'checked_out', 'c.checked_out',
				'checked_out_time', 'c.checked_out_time',
				'state', 'c.state',
				'access', 'c.access', 'access_level',
				'created', 'c.created',
				'created_by', 'c.created_by',
				'publish_up', 'c.publish_up',
				'publish_down', 'c.publish_down',
			);
		}

		parent::__construct($config);
	}
	
	/**
	 * Method to auto-populate the model state.
	 *
	 * Note. Calling getState in this method will result in recursion.
	 *
	 * @return	void
	 * @since	1.6
	 */
	protected function populateState($ordering = null, $direction = null)
	{
		// Initialise variables.
		$app = JFactory::getApplication();
		$session = JFactory::getSession();

		// Adjust the context to support modal layouts.
		if ($layout = JRequest::getVar('layout')) {
			$this->context .= '.'.$layout;
		}

		$search = $this->getUserStateFromRequest($this->context.'.filter.search', 'filter_search');
		$this->setState('filter.search', $search);

		$access = $this->getUserStateFromRequest($this->context.'.filter.access', 'filter_access', 0, 'int');
		$this->setState('filter.access', $access);

		$published = $this->getUserStateFromRequest($this->context.'.filter.published', 'filter_published', '');
		$this->setState('filter.published', $published);

		// List state information.
		parent::populateState('c.name', 'asc');
	}
	
	/**
	 * Build an SQL query to load the list data.
	 *
	 * @return	JDatabaseQuery
	 */
	protected function getListQuery()
	{
		// Create a new query object.
		$db = $this->getDbo();
		$query = $db->getQuery(true);
		
		// Select the required fields from the table.
		$query->select(
			$this->getState(
				'list.select',
				'c.id, c.name, c.alias, c.state, c.created, c.created_by, c.created_by_alias, c.checked_out, c.checked_out_time' .
				', c.modified, c.modified_by, c.access, c.publish_up, c.publish_down, c.home'
			)
		);
		$query->from('#__collector AS c');
		
		// Join over the users for the checked out user.
		$query->select('uc.name AS editor');
		$query->join('LEFT', '#__users AS uc ON uc.id = c.checked_out');
		
		// Join over the asset groups.
		$query->select('ag.title AS access_level');
		$query->join('LEFT', '#__viewlevels AS ag ON ag.id = c.access');
		
		// Join over the users for the author.
		$query->select('ua.name AS author_name');
		$query->join('LEFT', '#__users AS ua ON ua.id = c.created_by');
		
		// Filter by access level.
		if ($access = $this->getState('filter.access')) {
			$query->where('c.access = ' . (int) $access);
		}
		
		// Filter by published state
		$published = $this->getState('filter.published');
		if (is_numeric($published)) {
			$query->where('c.state = ' . (int) $published);
		}
		else if ($published === '') {
			$query->where('(c.state = 0 OR c.state = 1)');
		}
		
		// Filter by search in title.
		$search = $this->getState('filter.search');
		if (!empty($search)) {
			if (stripos($search, 'id:') === 0) {
				$query->where('c.id = '.(int) substr($search, 3));
			}
			else if (stripos($search, 'author:') === 0) {
				$search = $db->Quote('%'.$db->getEscaped(substr($search, 7), true).'%');
				$query->where('(ua.name LIKE '.$search.' OR ua.username LIKE '.$search.')');
			}
			else {
				$search = $db->Quote('%'.$db->getEscaped($search, true).'%');
				$query->where('(c.name LIKE '.$search.' OR c.alias LIKE '.$search.')');
			}
		}
		
		// Add the list ordering clause.
		$orderCol	= $this->state->get('list.ordering');
		$orderDirn	= $this->state->get('list.direction');
		$query->order($db->getEscaped($orderCol.' '.$orderDirn));
		
		return $query;
	}
	
	/**
	 * Method to get a list of collections.
	 * Overridden to add a check for access levels.
	 *
	 * @return	mixed	An array of data items on success, false on failure.
	 */
	/*public function getItems()
	{
		$items	= parent::getItems();
		$app	= JFactory::getApplication();
		if ($app->isSite()) {
			$user	= JFactory::getUser();
			$groups	= $user->getAuthorisedViewLevels();

			for ($x = 0, $count = count($items); $x < $count; $x++) {
				//Check the access level. Remove collections the user shouldn't see
				if (!in_array($items[$x]->access, $groups)) {
					unset($items[$x]);
				}
			}
		}
		return $items;
	}*/
}

}
