<?php
/**
 * Joomla! 1.5 component Collector
 *
 * @version 	$Id: folderlist.php 122 2011-09-13 22:26:20Z steevo $
 * @author 		Philippe Ousset steevo@steevo.fr
 * @package 	Collector
 * @license 	GNU/GPL
 *
 * Collector is a Multi Purpose Listing Tool.
 * Originaly developped to list Collections
 * it can be used for several purpose.
 */

// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die( 'Restricted access' );

jimport( 'joomla.application.component.model' );
jimport( 'joomla.filesystem.folder' );
jimport( 'joomla.filesystem.file' );

if(version_compare(JVERSION,'1.6.0','<')){
//Code pour Joomla! 1.5

/**
 * Folderlist model
 * @package	Collector
 */
class CollectorsModelFolderlist extends JModel
{
	/**
	 * The relative Path from site root to the current folder
	 * @var string 
	 */
	var $_folder;
	
	/**
	 * The path to the current folder
	 * @var array 
	 */
	var $_path;
	
	/**
	 * Folderlist model constructor
	 *
	 * @access	public
	 */
	function __construct()
	{
		parent::__construct();
		
		global $mainframe, $option;
		
        // Get folder request variables
		if(version_compare(JVERSION,'1.6.0','<')){
			//Code pour Joomla! 1.5
			$this->_folder = $mainframe->getUserStateFromRequest($option.'.filemanager.folder', 'folder', '', 'text');
		}else{
			//Code pour Joomla >= 1.6.0
			$app = JFactory::getApplication();
			$this->_folder = $app->getUserStateFromRequest($option.'.filemanager.folder', 'folder', '', 'text');
		}
		
		$this->_path = array();
		$tmp = new JObject();
		$tmp->name = JText::_( 'COM_COLLECTOR_ROOT' );
		$tmp->path = '';
		$this->_path[] = $tmp;
		
		if ( $this->_folder != '' )
		{
			$this->cleanPath();
		}
	}
	
	/**
	 * Retrieve the current folder
	 *
	 * @access	public
	 * @return	string	Relative Path from site root
	 */
	function getFolder()
	{
		return $this->_folder;
	}
	
	/**
	 * Retrieves list of folders and files from a folder
	 *
	 * @access	public
	 * @param	string	$baseFolder	Relative Path from site root
	 * @return	array				Array of objects containing the data from the filesystem
	 */
	function getList($baseFolder = null)
	{
		$baseSite = str_replace(DS, '/', JPATH_ROOT.DS);
		
		//Si aucun dossier specifie on va a la racine du site
		if ( $baseFolder == null )
		{
			if ( $this->_folder == '' )
			{
				$baseFolder = JPATH_ROOT.DS;
			}
			else
			{
				$baseFolder = JPATH_ROOT.DS.$this->_folder;
				
				$precFolder = implode ( '/' , explode( '/', $baseFolder, -1 ) );
				
				$prec = new JObject();
				$prec->name = JText::_('COM_COLLECTOR_BACK_DIRECTORY');
				$prec->type = '';
				$prec->ico = 'components/com_collector/assets/images/folder_back.png';
				$prec->path = str_replace(DS, '/', JPath::clean($precFolder));
				if ( $prec->path.'/' == $baseSite )
				{
					$prec->path_relative = '';
				}
				else
				{
					$prec->path_relative = str_replace($baseSite, '', $prec->path);
				}
				//$count = MediaHelper::countFiles($prec->path);
				//$prec->files = $count[0];
				//$prec->folders = $count[1];
				
				$folders[] = $prec;
			}
		}
		else
		{
			$baseFolder = JPATH_ROOT.DS.$baseFolder;
		}
		
		$foldersList = JFolder::folders($baseFolder);
		
		if ( $foldersList != null )
		{
			foreach ( $foldersList as $folder )
			{
				$tmp = new JObject();
				$tmp->name = basename($folder);
				$tmp->text = JText::_('COM_COLLECTOR_DIRECTORY');
				$tmp->ico = 'components/com_collector/assets/images/folder.png';
				$tmp->path = str_replace(DS, '/', JPath::clean($baseFolder.DS.$folder));
				$tmp->path_relative = str_replace($baseSite, '', $tmp->path);
				//$count = MediaHelper::countFiles($tmp->path);
				//$tmp->files = $count[0];
				//$tmp->folders = $count[1];
				
				$folders[] = $tmp;
			}
		}
		
		$this->list = array('folders' => $folders);
		
		return $this->list;
	}
	
	/**
	 * Check folder from request
	 * Clean <var>_folder</var>
	 *
	 * @access	public
	 * @return	void
	 */
	function cleanPath()
	{
		$this->_folder = str_replace('\\','/',$this->_folder);
		$path = explode('/',$this->_folder);
		
		$i = 0;
		
		foreach ( $path as $folder )
		{
			if ( $folder == '..' )
			{
				$i = $i - 1;
				if ( $i < 0 )
				{
					$cleanPath = array();
					$this->_path = array();
					$tmp = new JObject();
					$tmp->name = JText::_( 'COM_COLLECTOR_ROOT' );
					$tmp->path = '';
					$this->_path[] = $tmp;
					break;
				}
				else
				{
					unset($cleanPath[$i]);
					unset($this->_path[$i]);
				}
			}
			else
			{
				if ( $folder != '.' )
				{
					$cleanPath[] = $folder;
					$tmp = new JObject();
					$tmp->name = $folder;
					$tmp->path = implode('/', $cleanPath);
					$this->_path[] = $tmp;
					$i = $i + 1;
				}
			}
		}
		
		$this->_folder = implode('/', $cleanPath);
	}
	
	/**
	 * Construct Path links for navigation in the filesystem
	 *
	 * @access	public
	 * @return 	string 	Navigation html
	 */
	function get_path_navigation()
	{
		$navigation = '';
		
		foreach ($this->_path as $folder)
		{
			$navigation .= '<a href="' . JRoute::_( 'index.php?option=com_collector&amp;view=folderlist&amp;tmpl=component&amp;folder='.$folder->path ) . '" >' . $folder->name . '</a>/';
		}
		
		return $navigation;
	}
	
	/**
	 * Method to create folder
	 *
	 * @access	public
	 * @return	string	Message to display
	 */
	function createFolder()
	{
		$folder = JRequest::getVar('foldername', '', '', 'text');
		
		if (strlen($folder) > 0) {
			$path = JPath::clean(JPATH_ROOT.DS.$this->_folder.DS.$folder);
			if (!is_dir($path))
			{
				if (!is_file($path))
				{
					JFolder::create($path);
					JFile::write($path.DS."index.html", "<html><body bgcolor=\"#FFFFFF\"></body></html>");
					$msg[0] = '"' . $folder . '" ' . JText::_( 'COM_COLLECTOR_FOLDER_CREATED' );
					$msg[1] = 'message';
				}
				else
				{
					$msg[0] = JText::_( 'COM_COLLECTOR_EXIST_FILE_SAME' );
					$msg[1] = 'error';
				}
			}
			else
			{
				$msg[0] = JText::_( 'COM_COLLECTOR_EXIST_FOLDER_SAME' );
				$msg[1] = 'error';
			}
			
		}
		else
		{
			$msg[0] = JText::_( 'COM_COLLECTOR_NO_FOLDER_NAME' );
			$msg[1] = 'error';
		}
		
		return $msg;
	}
}

}else{

//Code pour Joomla >= 1.6.0
jimport('joomla.application.component.modellist');

/**
 * Folderlist model
 * @package	Collector
 */
class CollectorModelFolderlist extends JModelList
{
	/**
	 * The relative Path from site root to the current folder
	 * @var string 
	 */
	var $_folder;
	
	/**
	 * The path to the current folder
	 * @var array 
	 */
	var $_path;
	
	/**
	 * Folderlist model constructor
	 *
	 * @access	public
	 */
	function __construct()
	{
		parent::__construct();
		
		// Get folder request variables
		$application = JFactory::getApplication();
		$this->_folder = $application->getUserStateFromRequest('collector.filemanager.folder', 'folder', '', 'text');
		
		$this->_path = array();
		$tmp = new JObject();
		$tmp->name = JText::_( 'COM_COLLECTOR_ROOT' );
		$tmp->path = '';
		$this->_path[] = $tmp;
		
		if ( $this->_folder != '' )
		{
			$this->cleanPath();
		}
	}
	
	/**
	 * Retrieve the current folder
	 *
	 * @access	public
	 * @return	string	Relative Path from site root
	 */
	function getFolder()
	{
		return $this->_folder;
	}
	
	/**
	 * Retrieves list of folders and files from a folder
	 *
	 * @access	public
	 * @param	string	$baseFolder	Relative Path from site root
	 * @return	array				Array of objects containing the data from the filesystem
	 */
	function getList($baseFolder = null)
	{
		$baseSite = str_replace(DS, '/', JPATH_ROOT.DS);
		
		//Si aucun dossier specifie on va a la racine du site
		if ( $baseFolder == null )
		{
			if ( $this->_folder == '' )
			{
				$baseFolder = JPATH_ROOT.DS;
			}
			else
			{
				$baseFolder = JPATH_ROOT.DS.$this->_folder;
				
				$precFolder = implode ( '/' , explode( '/', $baseFolder, -1 ) );
				
				$prec = new JObject();
				$prec->name = JText::_('COM_COLLECTOR_BACK_DIRECTORY');
				$prec->type = '';
				$prec->ico = 'components/com_collector/assets/images/folder_back.png';
				$prec->path = str_replace(DS, '/', JPath::clean($precFolder));
				if ( $prec->path.'/' == $baseSite )
				{
					$prec->path_relative = '';
				}
				else
				{
					$prec->path_relative = str_replace($baseSite, '', $prec->path);
				}
				//$count = MediaHelper::countFiles($prec->path);
				//$prec->files = $count[0];
				//$prec->folders = $count[1];
				
				$folders[] = $prec;
			}
		}
		else
		{
			$baseFolder = JPATH_ROOT.DS.$baseFolder;
		}
		
		$foldersList = JFolder::folders($baseFolder);
		
		if ( $foldersList != null )
		{
			foreach ( $foldersList as $folder )
			{
				$tmp = new JObject();
				$tmp->name = basename($folder);
				$tmp->text = JText::_('COM_COLLECTOR_DIRECTORY');
				$tmp->ico = 'components/com_collector/assets/images/folder.png';
				$tmp->path = str_replace(DS, '/', JPath::clean($baseFolder.DS.$folder));
				$tmp->path_relative = str_replace($baseSite, '', $tmp->path);
				//$count = MediaHelper::countFiles($tmp->path);
				//$tmp->files = $count[0];
				//$tmp->folders = $count[1];
				
				$folders[] = $tmp;
			}
		}
		
		$this->list = array('folders' => $folders);
		
		return $this->list;
	}
	
	/**
	 * Check folder from request
	 * Clean <var>_folder</var>
	 *
	 * @access	public
	 * @return	void
	 */
	function cleanPath()
	{
		$this->_folder = str_replace('\\','/',$this->_folder);
		$path = explode('/',$this->_folder);
		
		$i = 0;
		
		foreach ( $path as $folder )
		{
			if ( $folder == '..' )
			{
				$i = $i - 1;
				if ( $i < 0 )
				{
					$cleanPath = array();
					$this->_path = array();
					$tmp = new JObject();
					$tmp->name = JText::_( 'COM_COLLECTOR_ROOT' );
					$tmp->path = '';
					$this->_path[] = $tmp;
					break;
				}
				else
				{
					unset($cleanPath[$i]);
					unset($this->_path[$i]);
				}
			}
			else
			{
				if ( $folder != '.' )
				{
					$cleanPath[] = $folder;
					$tmp = new JObject();
					$tmp->name = $folder;
					$tmp->path = implode('/', $cleanPath);
					$this->_path[] = $tmp;
					$i = $i + 1;
				}
			}
		}
		
		$this->_folder = implode('/', $cleanPath);
	}
	
	/**
	 * Construct Path links for navigation in the filesystem
	 *
	 * @access	public
	 * @return 	string 	Navigation html
	 */
	function get_path_navigation()
	{
		$navigation = '';
		
		foreach ($this->_path as $folder)
		{
			$navigation .= '<a href="' . JRoute::_( 'index.php?option=com_collector&amp;view=folderlist&amp;tmpl=component&amp;folder='.$folder->path ) . '" >' . $folder->name . '</a>/';
		}
		
		return $navigation;
	}
	
	/**
	 * Method to create folder
	 *
	 * @access	public
	 * @return	string	Message to display
	 */
	function createFolder()
	{
		$folder = JRequest::getVar('foldername', '', '', 'text');
		
		if (strlen($folder) > 0) {
			$path = JPath::clean(JPATH_ROOT.DS.$this->_folder.DS.$folder);
			if (!is_dir($path))
			{
				if (!is_file($path))
				{
					JFolder::create($path);
					JFile::write($path.DS."index.html", "<html><body bgcolor=\"#FFFFFF\"></body></html>");
					$msg[0] = '"' . $folder . '" ' . JText::_( 'COM_COLLECTOR_FOLDER_CREATED' );
					$msg[1] = 'message';
				}
				else
				{
					$msg[0] = JText::_( 'COM_COLLECTOR_EXIST_FILE_SAME' );
					$msg[1] = 'error';
				}
			}
			else
			{
				$msg[0] = JText::_( 'COM_COLLECTOR_EXIST_FOLDER_SAME' );
				$msg[1] = 'error';
			}
			
		}
		else
		{
			$msg[0] = JText::_( 'COM_COLLECTOR_NO_FOLDER_NAME' );
			$msg[1] = 'error';
		}
		
		return $msg;
	}
}

}