<?php
/**
 * Joomla! 1.5 component Collector
 *
 * @version 	$Id: itemedit.php 118 2011-04-21 22:19:54Z steevo $
 * @author 		Philippe Ousset steevo@steevo.fr
 * @package 	Collector
 * @license 	GNU/GPL
 *
 * Collector is a Multi Purpose Listing Tool.
 * Originaly developped to list Collections
 * it can be used for several purpose.
 */

// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die( 'Restricted access' );

jimport( 'joomla.application.component.model' );

/**
 * Itemedit model
 * @package	Collector
 */
class CollectorsModelItemedit extends JModel
{
	/**
	 * Collection Id
	 * @var int
	 */
	var $_collection = null;

	/**
	 * Item Id
	 * @var int
	 */
	var $_itemId = null;

	/**
	 * Default folder image storage
	 * @var string
	 */
	var $_folder = null;

	/**
	 * Items total
	 * @var int
	 */
	var $_total = null;

	/**
	 * Itemedit model constructor
	 *
	 * @access	public
	 */
	function __construct()
	{
		$this->_collection = JRequest::getVar( 'collection', '', '', 'int');
		$this->_itemId = JRequest::getVar( 'id', '', '', 'int');

		$db =& JFactory::getDBO();

		$query = 'SELECT alias';
		$query .= ' FROM #__collector';
		$query .= ' WHERE id = "' . $this->_collection . '"';

		$db->setQuery( $query );

		$this->_folder = 'images/collector/collection/'.$db->loadResult();

		parent::__construct();
	}

	/**
	 * Method to store a record
	 *
	 * @return	boolean	True on success
	 */
	function store()
	{
		global $mainframe;

		// on initialise les variables
		$params = &JComponentHelper::getParams( 'com_collector' );

		$db =& JFactory::getDBO();
		$user =& JFactory::getUser();
		$nulldate = $db->getNullDate();

		$item = & JTable::getInstance('collector_items','Table');
		
		if ( $this->_itemId != '' )
		{
			$item->loadVersion($this->_collection,$this->_itemId);
		}
		else
		{
			$item->initVersion($this->_collection);
		}

		if ( $params->get('save_history') )
		{
			$version = null;
		}
		else
		{
			$version = $item->history->id;
		}

		if (!$item->bindVersion(JRequest::get('post')))
		{
			JError::raiseError( 500, $db->stderr() );
			return false;
		}

		// sanitise id field
		$item->id = (int) $item->id;

		$post = JRequest::get('post');

		foreach ($post as $k => $v)
		{
			if (!is_array($k))
			{
				$data->$k = $v;
			}
		}

		$details = JRequest::getVar('details', array(0), 'post', 'array');

		if (is_array($details))
		{
			foreach ($details as $k => $v)
			{
				$data->$k = $v;
			}
		}

		$metadata = JRequest::getVar('meta', null, 'post', 'array');

		if (is_array($metadata))
		{
			$txt = array();
			foreach ($metadata as $k => $v)
			{
				if ($k == 'description')
				{
					$data->metadesc = $v;
				}
				else if ($k == 'keywords')
				{
					$data->metakey = $v;
				}
				else
				{
					$txt[] = "$k=$v";
				}
			}
			$data->metadata = implode("\n", $txt);
		}

		//horodatage universel
		$config =& JFactory::getConfig();
		$tzoffset = $config->getValue('config.offset');
		$date =& JFactory::getDate($data->created, $tzoffset);
		$data->created = $date->toMySQL();
		$date =& JFactory::getDate($data->publish_up, $tzoffset);
		$data->publish_up = $date->toMySQL();
		$date =& JFactory::getDate($data->publish_down, $tzoffset);
		$data->publish_down = $date->toMySQL();

		// bind the form fields to the collector table
		if ( !$item->bindVersion($data) )
		{
			JError::raiseError( 500, $item->getError() );
			return false;
		}

		// make sure the collector record is valid
		if ( !$item->checkVersion() )
		{
			JError::raiseWarning( 500, $item->getError() );
			return false;
		}

		// store the record in the database
		if ( !$item->storeVersion($version) )
		{
			JError::raiseError( 500, $item->getError() );
			return false;
		}

		// libere la collection
		$item->checkin();

		return true;
	}

	/**
	 * Method to get fields from a collection
	 *
	 * @access	public
	 * @param	int		$collection	Collection Id
	 * @return	array				Array of objects containing the fields from the database
	 */
	function getCollectionFields($collection)
	{
		$this->getDBO();

		$query = 'SELECT id, field, tablecolumn, type, defined, template';
		$query .= ' FROM `#__collector_fields`';
		$query .= ' WHERE collection = "' . $collection . '"';
		$query .= ' ORDER BY ordering';

		$this->_db->setQuery($query);

		$fields = $this->_db->loadObjectList();

		return $fields;
	}

	/**
	 * Method to get items from a collection
	 *
	 * @access	public
	 * @param	object	$field	Field associate to a collection
	 * @return	array			Array of objects containing the items from the database
	 */
	function getItemsList($field)
	{
		// recuperation du titre personnalise pour la collection
		$collection = $field->defined;

		$query = 'SELECT custom';
		$query .= ' FROM `#__collector`';
		$query .= ' WHERE id = \'' . $collection . '\'';

		$row = $this->_getList( $query );

		if ( $row[0]->custom == "0" )
		{
			$query = 'SELECT id';
			$query .= ' FROM `#__collector_fields`';
			$query .= ' WHERE collection = "' . $collection . '"';
			$query .= ' AND home = "1"';
			$row = $this->_getList( $query );
			$custom[] = $row[0]->id;
		}
		else
		{
			$custom = explode( '/', $row[0]->custom );
		}

		$query = 'SELECT id, field, type, defined';
		$query .= ' FROM `#__collector_fields`';
		$query .= ' WHERE collection = \'' . $collection . '\'';

		$results = $this->_getList( $query );

		$i = 0;

		foreach ( $custom as $field )
		{
			for ($j = 0; $j < count($results); $j ++)
			{
				$number_field_default[$i] = $field;

				if ( $results[$j]->id == $field )
				{
					$name_field_default[$i] = $results[$j]->field;
					$column_field_default[$i] = $results[$j]->tablecolumn;
					if ( $results[$j]->type == 3 )
					{
						$defined_field_default[$i] = $results[$j]->defined;
					}
					else
					{
						$defined_field_default[$i] = null;
					}
				}
			}
			$i++;
		}

		// recherche des objets de la collection
		//$query = 'SELECT c.id, c.state, ordering, hits, created, created_by, u.name AS author, created_by_alias, access, g.name AS groupname, publish_up, publish_down, ';
		$query = 'SELECT c.id';
		for ( $i=0; $i < count($number_field_default); $i ++ )
		{
			if ($defined_field_default[$i])
			{
				$query .= ', d'.$i.'.content AS name'.$i;
			}
			else
			{
				$query .= ', h.'.$column_field_default[$i].' AS name'.$i;
			}
		}
		$query .= ' FROM `#__collector_items` AS c';
		$query .= ' LEFT JOIN `#__users` AS u ON c.created_by = u.id';
		$query .= ' LEFT JOIN `#__groups` AS g ON c.access = g.id';
		$query .= ' LEFT JOIN `#__collector_items_history_'.$this->_collection.'` AS h ON h.item = c.id';
		for ( $i=0; $i < count($number_field_default); $i ++ )
		{
			if ($defined_field_default[$i])
			{
				$query .= ' LEFT JOIN `#__collector_defined_content` AS d'.$i.' ON d'.$i.'.id = h.'.$column_field_default[$i];
			}
		}

		$query .= ' WHERE h.state = 1';
		$query .= ' AND c.collection = '.$this->_collection;
		$query .= ' ORDER BY c.ordering';

		$results = $this->_getList( $query );

		foreach ( $results as $row )
		{
			$item = new stdClass;
			$item->value = $row->id;

			$name = $row->name0;

			for ($j=1; $j<count($number_field_default); $j ++)
			{
				$field = 'name'.$j;
				$name .= ' ' . $row->$field;
			}

			$item->text = $name;

			$items[] = $item;
		}

		return $items;
	}

	/**
	 * Method to checkin a record
	 *
	 * @access	public
	 * @return	boolean	True on success
	 */
	function checkin()
	{
		// on initialise les variables
		$db =& JFactory::getDBO();
		$user =& JFactory::getUser();

		$row = & JTable::getInstance('collector_items','Table');
		$row->initVersion($this->_collection);
		
		if (!$row->bind(JRequest::get('post'))) {
			JError::raiseError( 500, $db->stderr() );
			return false;
		}

		// libere la collection
		$row->checkin();

		return true;
	}

	/**
	 * Retrieve the default folder of storage
	 *
	 * @access	public
	 * @return	string	Relative Path from site root
	 */
	function getFolder()
	{
		return $this->_folder;
	}

	/**
	 * Method to reset Hit count of an item
	 *
	 * @access	public
	 * @param	int		$id	Item Id
	 * @return	boolean		True on success
	 */
	function resetHits($id)
	{
		global $mainframe;

		// Initialize variables
		$db	= & JFactory::getDBO();

		// Instantiate and load an article table
		$row =& $this->getTable('collector_items');
		$row->initVersion($this->_collection);
		
		$row->load($id);
		$row->hits = 0;
		if (!$row->store())
		{
			return false;
		}
		$row->checkin();

		return true;
	}
}
