<?php
/**
 * Joomla! 1.5 component Collector
 *
 * @version 	$Id: itemversions.php 127 2011-12-10 22:51:55Z steevo $
 * @author 		Philippe Ousset steevo@steevo.fr
 * @package 	Collector
 * @license 	GNU/GPL
 *
 * Collector is a Multi Purpose Listing Tool.
 * Originaly developped to list Collections
 * it can be used for several purpose.
 */

// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die( 'Restricted access' );

jimport( 'joomla.application.component.modellist' );

/**
 * Itemversions model
 * @package	Collector
 */
class CollectorModelItemversions extends JModelList
{
	/**
	 * Collection Id
	 * @var int
	 */
	var $_collection;

	/**
	 * Constructor.
	 *
	 * @param	array	An optional associative array of configuration settings.
	 * @see		JController
	 * @since	1.6
	 */
	public function __construct($config = array())
	{
		if (empty($config['filter_fields'])) {
			$config['filter_fields'] = array(
				'id', 'h.id',
				'item', 'h.alias',
				'state', 'h.state',
				'modified', 'h.modified',
				'modified_by', 'h.modified_by',
				'modification', 'h.modification',
			);
		}

		parent::__construct($config);
	}
	
	/**
	 * Method to auto-populate the model state.
	 *
	 * Note. Calling getState in this method will result in recursion.
	 *
	 * @return	void
	 * @since	1.6
	 */
	protected function populateState($ordering = null, $direction = null)
	{
		// Initialise variables.
		$app = JFactory::getApplication();
		$session = JFactory::getSession();

		// Adjust the context to support modal layouts.
		if ($layout = JRequest::getVar('layout')) {
			$this->context .= '.'.$layout;
		}

		$search = $this->getUserStateFromRequest($this->context.'.filter.search', 'filter_search');
		$this->setState('filter.search', $search);

		// List state information.
		parent::populateState('h.modified', 'desc');
	}
	
	/**
	 * Build an SQL query to load the list data.
	 *
	 * @return	JDatabaseQuery
	 */
	protected function getListQuery()
	{
		// Get collection
		$collection = $this->getCollection();
		$item = $this->getItem();
		
		// Create a new query object.
		$db = $this->getDbo();
		$query = $db->getQuery(true);
		
		// Select the required fields from the table.
		$query->select(
			$this->getState(
				'list.select',
				'h.id, h.item, h.state, h.modified, h.modified_by' .
				', h.modification'
			)
		);
		$query->from('#__collector_items_history_'.$collection->id.' AS h');
		
		// Join over the users for the author.
		$query->select('ua.name AS author_name');
		$query->join('LEFT', '#__users AS ua ON ua.id = h.modified_by');
		
		// Filter by item
		$query->where('h.item = '.$item->id);
		
		// Filter by search in title.
		$search = $this->getState('filter.search');
		if (!empty($search)) {
			if (stripos($search, 'id:') === 0) {
				$query->where('h.id = '.(int) substr($search, 3));
			}
			else if (stripos($search, 'author:') === 0) {
				$search = $db->Quote('%'.$db->getEscaped(substr($search, 7), true).'%');
				$query->where('(ua.name LIKE '.$search.' OR ua.username LIKE '.$search.')');
			}
			else {
				$search = $db->Quote('%'.$db->getEscaped($search, true).'%');
				$query->where('(h.modification LIKE '.$search.')');
			}
		}
		
		// Add the list ordering clause.
		$orderCol	= $this->state->get('list.ordering');
		$orderDirn	= $this->state->get('list.direction');
		$query->order($db->getEscaped($orderCol.' '.$orderDirn));
		
		return $query;
	}
	
	/**
	 * Method to load default collection Id in <var>_collection</var>
	 *
	 * @access	private
	 */
	public function getCollection()
	{
		if ( empty($this->_collection) )
		{
			$collection = JRequest::getVar( 'collection', '', '', 'int');
			
			if ($collection == '')
			{
				$query = 'SELECT id';
				$query .= ' FROM `#__collector`';
				$query .= ' WHERE home = 1';
				
				$row = $this->_getList( $query );
				
				if ( !$row )
				{
					/**
					 * tag  defaut d'une collection
					 */
					$query = 'UPDATE `#__collector` SET home = 1 LIMIT 1;';
					$this->_db->setQuery( $query );
					$this->_db->query();
					
					$query = 'SELECT id';
					$query .= ' FROM `#__collector`';
					$query .= ' WHERE home = 1';
					
					$row = $this->_getList( $query );
				}
				$collection = $row[0]->id;
			}
			
			// Create a new query object.
			$db =& JFactory::getDBO();
			$query = $db->getQuery(true);
			
			// Select the required fields from the table.
			$query->select('id, name, custom');
			$query->from('#__collector');
			
			// Add the filder on ID
			$query->where('id = '.$collection);
			
			$db->setQuery( $query );
			
			$this->_collection = $db->loadObject();
		}
		
		return $this->_collection;
	}

	/**
	 * Method to load default collection Id in <var>_collection</var>
	 *
	 * @access	private
	 */
	public function getItem()
	{
		if ( empty($this->_item) )
		{
			$item = JRequest::getVar( 'item', '', '', 'int');
			
			// Create a new query object.
			$db =& JFactory::getDBO();
			$query = $db->getQuery(true);
			
			// Select the required fields from the table.
			$query->select('id, alias');
			$query->from('#__collector_items');
			
			// Add the filder on ID
			$query->where('id = '.$item);
			
			$db->setQuery( $query );
			
			$this->_item = $db->loadObject();
		}
		
		return $this->_item;
	}

	/**
	 * Method to load listdrop of collections
	 *
	 * @access	public
	 */
	public function getCollections()
	{
		$collection = $this->getCollection();
		
		// Create a new query object.
		$db =& JFactory::getDBO();
		$query = $db->getQuery(true);
		
		// Select the required fields from the table.
		$query->select('id AS value, name AS text');
		$query->from('#__collector');
		
		// Add the list ordering clause.
		$query->order('text');
		
		$db->setQuery( $query );
		
		$results = $db->loadObjectList();
		
		return JHTML::_('select.genericlist', $results, 'collection', ' class="inputbox" size="1" onchange="this.form.submit()" disabled=disabled', 'value', 'text', $collection->id);
	}
	
	/**
	 * Retrieves item details
	 * Put data in <var>_item</var>
	 * Returns a reference to the global {@link TableCollector_items} object
	 *
	 * @access	public
	 * @return	object TableCollector_items
	 */
	// function &getItem()
	// {
		// if ( empty( $this->_item ))
		// {
			// $this->_item = & JTable::getInstance('collector_items','Table');
			// $this->_item->initVersion($this->_collection);
			// $this->_item->load($this->_id);
		// }
		
		// return $this->_item;
	// }
	
	/**
	 * Method to delete defined items
	 *
	 * @access	public
	 * @return	string	Message to display
	 */
	function delete($item,$cid)
	{
		$row =& $this->getTable('collector_items');
		
		$row->load($item);

		foreach ($cid as $id)
		{
			if (!($row->deleteVersion($id)))
			{
				JError::raiseError( 500, $row->getError() );
				return false;
			}
		}
		
		return true;
	}
}
