<?php
/**
 * Joomla! 1.5 component Collector
 *
 * @version 	$Id: collection.php 130 2012-02-19 20:14:53Z steevo $
 * @author 		Philippe Ousset steevo@steevo.fr
 * @package 	Collector
 * @license 	GNU/GPL
 *
 * Collector is a Multi Purpose Listing Tool.
 * Originaly developped to list Collections
 * it can be used for several purpose.
 */

// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die( 'Restricted access' );


if(version_compare(JVERSION,'1.6.0','<')){
//Code pour Joomla! 1.5
jimport( 'joomla.application.component.model' );
/**
 * Collection model
 * @package	Collector
 */
class CollectorModelCollection extends JModel
{
	/**
	 * Collection data
	 * @var array object
	 */
	var $_collection = null;
	
	/**
	 * Collection template
	 * @var object
	 */
	var $_template = null;
	
	/**
	 * To know if there is a research
	 * @var boolean
	 */
	var $_search = 0;
	
	/**
	 * Number of results
	 * @var int
	 */
	var $_total = 0;
	
	/**
	 * Reference to the global {@link JPagination} object
	 * @var object JPagination
	 */
	var $_pagination = 0;
	
	/**
	 * Collection model constructor
	 * @access	public
	 */
	function __construct()
	{
		parent::__construct();
		
		$id = JRequest::getVar('id', 0, '', 'int');
		$this->setId((int)$id);
		
		global $mainframe, $option;
		
		// Get the pagination request variables
		if(version_compare(JVERSION,'1.6.0','<')){
			//Code pour Joomla! 1.5
			$limitstart	= $mainframe->getUserStateFromRequest($option.'.collection.limitstart', 'limitstart', 0, 'int');
			$limit = $mainframe->getUserStateFromRequest($option.'.collection.limit', 'limit', $mainframe->getCfg('list_limit'), 'int');
		}else{
			//Code pour Joomla >= 1.6.0
			$app = JFactory::getApplication();
			$limitstart	= $app->getUserStateFromRequest($option.'.collection.limitstart', 'limitstart', 0, 'int');
			$limit = $app->getUserStateFromRequest($option.'.collection.limit', 'limit', $mainframe->getCfg('list_limit'), 'int');
		}
		
		// In case limit has been changed, adjust it
		$limitstart = ($limit != 0 ? (floor($limitstart / $limit) * $limit) : 0);
		$mainframe->setUserState($option.'.collection.limitstart',0);
		
		$this->setState('limit', $limit);
		$this->setState('limitstart', $limitstart);
	}
	
	/**
	 * Method to set the collection id
	 *
	 * @access	public
	 * @param	int	Collection ID number
	 */
	function setId($id)
	{
		// Set new collection ID and wipe data
		$this->_id = $id;
		$this->_collection = null;
	}
	
	/**
	 * Method to load collection data
	 * Put data in <var>_collection</var>
	 *
	 * @access	private
	 * @return	boolean	True on success
	 */
	function _loadCollection()
	{
		global $mainframe;
		
		if($this->_id == '0')
		{
			return false;
		}
		
		// Load the collection if it doesn't already exist
		if (empty($this->_collection))
		{
			// Get the page/component configuration
			$query = 'SELECT c.*, u.name AS author,'.
					' CASE WHEN CHAR_LENGTH(c.alias) THEN CONCAT_WS(\':\', c.id, c.alias) ELSE c.id END as slug'.
					' FROM #__collector AS c'.
					' LEFT JOIN #__users AS u ON u.id = c.created_by' .
					' WHERE c.id = '. (int) $this->_id;
			$this->_db->setQuery($query);
			$this->_collection = $this->_db->loadObject();
			
			if ( ! $this->_collection ) {
				return false;
			}
			if($this->_collection->publish_down == $this->_db->getNullDate()) {
				//$this->_collection->publish_down = JText::_('COM_COLLECTOR_NEVER');
			}
		}
		
		return true;
	}


	/**
	 * Method to get collection data
	 *
	 * @access	public
	 * @return	object	Collection informations
	 */
	function &getCollection()
	{
		global $mainframe, $option;
		
		$reset = $mainframe->getUserState($option.'.collection.reset');
		
		// Load the collection data
		if ($this->_loadCollection())
		{
			// Is the collection published?
			if (!$this->isPublished())
			{
				JError::raiseError( 404, JText::_('COM_COLLECTOR_COLLECTION_NOT_PUBLISHED') );
			}
			
			$this->_collection->parameters =& $this->_loadCollectionParams();
			$this->_collection->fieldsDisplayed = $this->_loadCollectionFields(false);
			$this->_collection->fields = $this->_loadCollectionFields(true);
			
			if ( $this->_collection->custom != '0' )
			{
				$this->custom = explode( '/', $this->_collection->custom );
			}
			else
			{
				foreach ( $this->_collection->fields as $field )
				{
					if ( $field->home == 1 )
					{
						$this->custom[0] = $field->id;
						break;
					}
				}
			}
			
			if ( $reset )
			{
				$this->searchReset();
			}
			$this->_collection->items = $this->_loadItems($reset);
		}
		else
		{
			$collection =& JTable::getInstance('collector');
			$this->_collection = $collection;
		}
		
		return $this->_collection;
	}
	
	/**
	 * Method to load collection parameters
	 * Returns a reference to the global {@link JParameter} object
	 *
	 * @access	private
	 * @return	object JParameter
	 */
	function &_loadCollectionParams()
	{
		global $mainframe;
		
		// Get the page/component configuration
		$params = clone($mainframe->getParams('com_collector'));
		
		// Return the collection object's parameters
		return $params;
	}
	
	/**
	 * Method to load fields informations
	 *
	 * @access	private
	 * @param	boolean	$search	True if there is a research
	 * @return	mixed			Array of fields objects. False if no fields loaded.
	 */
	function _loadCollectionFields( $search=false )
	{
		if ( $this->_id == '0' )
		{
			return false;
		}
		else
		{
			$collection = $this->_id;
		}
		
		$user		=& JFactory::getUser();
		$aid		= (int) $user->get('aid', 0);
		
		$jnow		=& JFactory::getDate();
		$now		= $jnow->toMySQL();
		$nullDate	= $this->_db->getNullDate();
		
		$query = 'SELECT c.*, u.name AS author, u.usertype';
		$query .= ' FROM #__collector_fields AS c';
		$query .= ' LEFT JOIN #__users AS u ON u.id = c.created_by';
		$query .= ' WHERE collection = ' . $collection;
		if ( $search == false )
		{
			$query .= ' AND c.listing = 1';
		}
		$query .= ' AND c.access <= '. (int) $aid;
		
		$query .= ' AND ( ';
		$query .= ' ( c.created_by = ' . (int) $user->id . ' ) ';
		$query .= '   OR ';
		$query .= ' ( c.state = 1' .
				' AND ( c.publish_up = '.$this->_db->Quote($nullDate).' OR c.publish_up <= '.$this->_db->Quote($now).' )' .
				' AND ( c.publish_down = '.$this->_db->Quote($nullDate).' OR c.publish_down >= '.$this->_db->Quote($now).' )';
		$query .= '   ) ';
		$query .= ' ) ';
				
		$query .= ' ORDER BY ordering';
		
		$this->_db->setQuery($query);
		$fields = $this->_db->loadObjectList();
		
		if ( ! $fields ) {
			return false;
		}
		
		return $fields;
	}
	
	/**
	 * Method to load items data
	 *
	 * @access	private
	 * @param	boolean	$reset	True is search is canceled
	 * @return	mixed			Array of items objects. False if no fields loaded
	 */
	function _loadItems($reset)
	{
		$model = JModel::getInstance('collection','CollectorModel');
		if($this->_id == '0')
		{
			return false;
		}
		
		$query = $this->_buildQuery();
		
		$this->setBigSelects();
		
		if ($reset)
		{
			$cache = & JFactory::getCache("com_collector");
			$items = $cache->call( array( $model, '_getList' ), $query, $this->getState('limitstart'), $this->getState('limit') );
			//$items = CollectorModelCollection::_getList( $query, $this->getState('limitstart'), $this->getState('limit') );
		}
		else
		{
			$items = $this->_getList( $query, $this->getState('limitstart'), $this->getState('limit') );
		}
		
		if ( ! $items ) {
			return false;
		}
		
		return $items;
	}
	
	/**
	 * Method to create the query to select items
	 *
	 * @access	private
	 * @param	boolean	$reset	True is search is canceled
	 * @return	string			Query to select items
	 */
	function _buildQuery($reset=0)
	{
		global $mainframe;
		
		$model = JModel::getInstance('collection','CollectorModel');
		
		$fields = $this->_collection->fields;
		
		// Get the WHERE and ORDER BY clauses for the query
		$where		= $this->_buildCollectionWhere();
		$orderby	= $this->_buildCollectionOrderBy();
		
		$query = 'SELECT c.id AS cid, c.*, u.name AS creator, u2.name AS author, u.usertype, h.*, ';
		
		foreach ($fields as $field)
		{
			if ($field->type == 3)
			{
				$query .= 'd'.$field->id.'.content AS `'.$field->tablecolumn.'_content`, ';
			}
		}
		$query .= ' CASE WHEN CHAR_LENGTH(c.alias) THEN CONCAT_WS(\':\', c.id, c.alias) ELSE c.id END as slug';
		$query .= ' FROM `#__collector_items` AS c';
		$query .= ' LEFT JOIN `#__users` AS u ON u.id = c.created_by';
		$query .= ' LEFT JOIN `#__users` AS u2 ON u2.id = c.modified_by';
		$query .= ' LEFT JOIN `#__collector_items_history_'.$this->_id.'` AS h ON h.item = c.id';
		
		foreach ($fields as $field)
		{
			if ($field->type == 3)
			{
				$query .= ' LEFT JOIN `#__collector_defined_content` AS d'.$field->id.' ON d'.$field->id.'.id = h.'.$field->tablecolumn;
			}
		}
		$query .= $where;
		$query .= $orderby;
		
		$this->setBigSelects();
		
		if (!$reset)
		{
			$cache = & JFactory::getCache("com_collector");
			$this->_total = $cache->call( array ( $model, '_getListCount' ), $query );
			//$this->_total = 7961;
		}
		else
		{
			$this->_total = $this->_getListCount($query);
		}
		
		return $query;
	}
	
	/**
	 * Method to build the WHERE clause of the query to select items
	 *
	 * @access	private
	 * @return	string	WHERE clause
	 */
	function _buildCollectionWhere()
	{
		global $mainframe, $option;
		
		$reset = $mainframe->getUserState($option.'.collection.reset');
		
		$fields = $this->_collection->fields;
		$params = $this->_collection->parameters;
		
		$user		=& JFactory::getUser();
		$aid		= (int) $user->get('aid', 0);
		
		$jnow		=& JFactory::getDate();
		$now		= $jnow->toMySQL();
		$nullDate	= $this->_db->getNullDate();
		
		/*
		 * First thing we need to do is assert that the content article is the one
		 * we are looking for and we have access to it.
		 */
		
		if(version_compare(JVERSION,'1.6.0','<')){
			//Code pour Joomla! 1.5
			$search_all_value = $reset ? '' : $mainframe->getUserStateFromRequest('collection.filter_search_all', 'filter_search_all');
		}else{
			//Code pour Joomla >= 1.6.0
			$app = JFactory::getApplication();
			$search_all_value = $reset ? '' : $app->getUserStateFromRequest('collection.filter_search_all', 'filter_search_all');
		}
		
		$where = ' WHERE h.state = 1';
		
		$where .= ' AND c.collection = '.$this->_id;
		
		$where .= ' AND c.access <= '. (int) $aid;
		
		$where .= ' AND ( ';
		$where .= ' ( c.created_by = ' . (int) $user->id . ' ) ';
		$where .= '   OR ';
		$where .= ' ( c.state = 1' .
				' AND ( c.publish_up = '.$this->_db->Quote($nullDate).' OR c.publish_up <= '.$this->_db->Quote($now).' )' .
				' AND ( c.publish_down = '.$this->_db->Quote($nullDate).' OR c.publish_down >= '.$this->_db->Quote($now).' )';
		$where .= '   ) ';
		$where .= ' ) ';
		
		$where2 = array();
		
		foreach ($fields as $field)
		{
			if ( $field->type == 3 )
			{
				$nameFilterCollection = 'filterfield_'.$field->id;
				$valueFilterCollection = $this->_collection->parameters->get( $nameFilterCollection, 0 );
				
				if(version_compare(JVERSION,'1.6.0','<')){
					//Code pour Joomla! 1.5
					$filter_field = $reset ? $valueFilterCollection : $mainframe->getUserStateFromRequest( $option.'.collection.filter_field_'.$field->id, 'filter_field_'.$field->id, '', 'int' );
				}else{
					//Code pour Joomla >= 1.6.0
					$app = JFactory::getApplication();
					$filter_field = $reset ? $valueFilterCollection : $mainframe->app( $option.'.collection.filter_field_'.$field->id, 'filter_field_'.$field->id, '', 'int' );
				}
				
				if ( $filter_field != '' )
				{
					$this->_search = 1;
					$where .= ' AND h.'.$field->tablecolumn.' = "'.$filter_field.'"';
				}
			}
			if ( $search_all_value != '' )
			{
				$this->_search = 1;
				if ( $field->type == 3 )
				{
					$where2[] = 'd'.$field->id.'.content LIKE "%'.$search_all_value.'%"';
				}
				else
				{
					$where2[] = 'h.'.$field->tablecolumn.' LIKE "%'.$search_all_value.'%"';
				}
			}
		}
		
		if (count($where2) > 0 ) {
			$where.= ' AND ('.implode(' OR ', $where2).')';
		}
		
		return $where;
	}
	
	/**
	 * Method to build the ORDER BY clause of the query to select items
	 *
	 * @access	private
	 * @return	string	ORDER BY clause
	 */
	function _buildCollectionOrderBy()
	{
		global $mainframe, $option;
		
		$reset = $mainframe->getUserState($option.'.collection.reset');
		
		$fields_type = array();
		
		$order = '';
		
		$params = $this->_collection->parameters;
		
		$special_order		= $params->get('order');
		if ( $special_order == 'ordering' ) {
			$this->_collection->order_default = 'c.ordering';
			$this->_collection->order_dir_default = '';
		} else if ( $special_order == 'created' ) {
			$this->_collection->order_default = 'c.created';
			$this->_collection->order_dir_default = '';
		} else if ( $special_order == 'rcreated' ) {
			$this->_collection->order_default = 'c.created';
			$this->_collection->order_dir_default = 'desc';
		} else if ( $special_order == 'modified' ) {
			$this->_collection->order_default = 'h.modified';
			$this->_collection->order_dir_default = 'desc';
		} else if ( $special_order == 'rmodified' ) {
			$this->_collection->order_default = 'h.modified';
			$this->_collection->order_dir_default = '';
		} else if ( $special_order == 'default' ) {
			foreach ($this->_collection->fields as $field)
			{
				$fields_type[$field->id] = $field->type;
				$fields_column[$field->id] = $field->tablecolumn;
			}
			foreach ( $this->custom as $field )
			{
				if ( $order != '' )
				{
					$order .= ' , ';
				}
				if ( $fields_type[$field] == 3 )
				{
					$order .= 'd'.$field.'.content';
				}
				else if ( $fields_type[$field] == 6 )
				{
					$order .= 'CAST(h.'. $fields_column[$field] .' AS SIGNED)';
				}
				else
				{
					$order .= 'h.'.$fields_column[$field];
				}
			}
			$this->_collection->order_default = $order;
			$this->_collection->order_dir_default = '';
		} else {
			$this->_collection->order_default = 'c.ordering';
			$this->_collection->order_dir_default = '';
		}
		
		if(version_compare(JVERSION,'1.6.0','<')){
			//Code pour Joomla! 1.5
			$this->_collection->filter_order		= $reset ? $this->_collection->order_default : $mainframe->getUserStateFromRequest( $option.'.collection.filter_order'.$this->_id, 'filter_order', '', 'string' );
			$this->_collection->filter_order_Dir	= $reset ? $this->_collection->order_dir_default : $mainframe->getUserStateFromRequest( $option.'.collection.filter_order_Dir'.$this->_id, 'filter_order_Dir', '', 'word' );
			//$filter_order		= $mainframe->getUserStateFromRequest( $option.'.collection.filter_order'.$this->_id, 'filter_order', '', 'string' );
			//$filter_order_Dir	= $mainframe->getUserStateFromRequest( $option.'.collection.filter_order_Dir'.$this->_id, 'filter_order_Dir', '', 'word' );
		}else{
			//Code pour Joomla >= 1.6.0
			$app = JFactory::getApplication();
			$this->_collection->filter_order		= $reset ? $this->_collection->order_default : $app->getUserStateFromRequest( $option.'.collection.filter_order'.$this->_id, 'filter_order', '', 'string' );
			$this->_collection->filter_order_Dir	= $reset ? $this->_collection->order_dir_default : $app->getUserStateFromRequest( $option.'.collection.filter_order_Dir'.$this->_id, 'filter_order_Dir', '', 'word' );
			//$filter_order		= $app->getUserStateFromRequest( $option.'.collection.filter_order'.$this->_id, 'filter_order', '', 'string' );
			//$filter_order_Dir	= $app->getUserStateFromRequest( $option.'.collection.filter_order_Dir'.$this->_id, 'filter_order_Dir', '', 'word' );
		}
			
		if ($this->_collection->filter_order == 'c.ordering'){
			$orderby 	= ' ORDER BY c.ordering '.$this->_collection->filter_order_Dir;
		} else {
			$orderby 	= ' ORDER BY '.$this->_collection->filter_order.' '.$this->_collection->filter_order_Dir.' , c.ordering ';
		}
		
		return $orderby;
	}
	
	/**
	 * Overridden get method to get properties from the collection
	 *
	 * @access	public
	 * @param	string	$property	The name of the property
	 * @param	mixed	$value		The value of the property to set
	 * @return 	mixed 				The value of the property
	 */
	function get($property, $default=null)
	{
		if ($this->_loadCollection()) {
			if(isset($this->_collection->$property)) {
				return $this->_collection->$property;
			}
		}
		return $default;
	}
	
	/**
	 * Method to get an item from a collection
	 *
	 * @access	public
	 * @param	object	$field	The field object
	 * @param	int		$id		The Id of the item
	 * @return 	mixed 			Item object. False if no item loaded
	 */
	function getObject($field, $id)
	{
		// recuperation du titre personnalise pour la collection
		$collection = $field->defined;
		
		$query = 'SELECT custom';
		$query .= ' FROM `#__collector`';
		$query .= ' WHERE id = \'' . $collection . '\'';
		
		$row = $this->_getList( $query );
		
		if ( $row[0]->custom == "0" )
		{
			$query = 'SELECT id';
			$query .= ' FROM `#__collector_fields`';
			$query .= ' WHERE collection = \'' . $collection . '\'';
			$query .= ' AND home = \'1\'';
			
			$row = $this->_getList( $query );
			
			$custom[] = $row[0]->id;
		}
		else
		{
			$custom = explode( '/', $row[0]->custom );
		}
		
		$query = 'SELECT id, field, tablecolumn, type, defined';
		$query .= ' FROM `#__collector_fields`';
		$query .= ' WHERE collection = \'' . $collection . '\'';
		
		$results = $this->_getList( $query );
		
		$i = 0;
		
		foreach ( $custom as $field )
		{
			for ($j = 0; $j < count($results); $j ++)
			{
				$number_field_default[$i] = $field;
				
				if ( $results[$j]->id == $field )
				{
					$name_field_default[$i] = $results[$j]->field;
					$column_field_default[$i] = $results[$j]->tablecolumn;
					if ( $results[$j]->type == 3 )
					{
						$defined_field_default[$i] = $results[$j]->defined;
					}
					else
					{
						$defined_field_default[$i] = null;
					}
				}
			}
			$i ++;
		}
		
		// recherche des objets de la collection
		//$query = 'SELECT c.id, c.state, ordering, hits, created, created_by, u.name AS author, created_by_alias, access, g.name AS groupname, publish_up, publish_down, ';
		$query = 'SELECT c.id,';
		for ( $i=0; $i < count($number_field_default); $i ++ )
		{
			if ($defined_field_default[$i])
			{
				$query .= ' d'.$i.'.content AS name'.$i.',';
			}
			else
			{
				$query .= ' h.'.$column_field_default[$i].' AS name'.$i.',';
			}
		}
		$query .= ' CASE WHEN CHAR_LENGTH(c.alias) THEN CONCAT_WS(\':\', c.id, c.alias) ELSE c.id END as slug';
		$query .= ' FROM `#__collector_items` AS c';
		$query .= ' LEFT JOIN `#__users` AS u ON c.created_by = u.id';
		$query .= ' LEFT JOIN `#__groups` AS g ON c.access = g.id';
		$query .= ' LEFT JOIN `#__collector_items_history_' . $collection . '` AS h ON h.item = c.id';
		for ( $i=0; $i < count($number_field_default); $i ++ )
		{
			if ($defined_field_default[$i])
			{
				$query .= ' LEFT JOIN `#__collector_defined_content` AS d'.$i.' ON h.' . $column_field_default[$i] . ' = d'.$i.'.id';
			}
		}
		$query .= ' WHERE h.state = 1';
		$query .= ' AND c.collection = \'' . $collection . '\'';
		$query .= ' AND c.id = \'' . $id . '\'';
		$query .= ' ORDER BY c.ordering';
		
		$this->_db->setQuery($query);
		$row = $this->_db->loadObject();
		
		if ($row == null)
		{
			return false;
		}
		$name = $row->name0;
		
		for ($j=1; $j<count($number_field_default); $j ++)
		{
			$field = 'name'.$j;
			$name .= ' ' . $row->$field;
		}
		
		$row->name = $name;
		
		return $row;
	}

	/**
	 * Tests if collection is filtered
	 *
	 * @access	public
	 * @return	boolean	True if filtered
	 */
	function isSearched()
	{
		if ( $this->_search == 1 )
		{
			return true;
		}
		
		return false;
	}
	
	/**
	 * Tests if collection is filtered
	 *
	 * @access	public
	 * @return	boolean	True if filtered
	 */
	function isFiltered()
	{
		$fields = $this->_collection->fields;
		foreach ($fields as $field)
		{
			if ( $field->type == 3 )
			{
				$nameFilterCollection = 'filterfield_'.$field->id;
				$valueFilterCollection = $this->_collection->parameters->get( $nameFilterCollection, 0 );
				
				if ( $valueFilterCollection )
				{
					return true;
				}
			}
		}
		return false;
	}
	
	/**
	 * Reset search values
	 *
	 * @access	public
	 * @return	boolean	True
	 */
	function searchReset()
	{
		global $mainframe, $option;
		
		$mainframe->setUserState('collection.filter_search_all', '');
		
		$fields = $this->_collection->fields;
		foreach ($fields as $field)
		{
			if ( $field->type == 3 )
			{
				$nameFilterCollection = 'filterfield_'.$field->id;
				$valueFilterCollection = $this->_collection->parameters->get( $nameFilterCollection, 0 );
				
				$mainframe->setUserState( $option.'.collection.filter_field_'.$field->id, $valueFilterCollection );
			}
		}
		
		$order = '';
		$fields_type = array();
		
		$special_order = $this->_collection->parameters->get('order');
		
		if ( $special_order == 'ordering' ) {
			$this->_collection->order_default = 'c.ordering';
			$this->_collection->order_dir_default = '';
		} else if ( $special_order == 'created' ) {
			$this->_collection->order_default = 'c.created';
			$this->_collection->order_dir_default = '';
		} else if ( $special_order == 'rcreated' ) {
			$this->_collection->order_default = 'c.created';
			$this->_collection->order_dir_default = 'desc';
		} else if ( $special_order == 'modified' ) {
			$this->_collection->order_default = 'h.modified';
			$this->_collection->order_dir_default = 'desc';
		} else if ( $special_order == 'rmodified' ) {
			$this->_collection->order_default = 'h.modified';
			$this->_collection->order_dir_default = '';
		} else if ( $special_order == 'default' ) {
			foreach ($fields as $field)
			{
				$fields_type[$field->id] = $field->type;
				$fields_column[$field->id] = $field->tablecolumn;
			}
			foreach ( $this->custom as $field )
			{
				if ( $order != '' )
				{
					$order .= ' , ';
				}
				if ( $fields_type[$field] == 3 )
				{
					$order .= 'd'.$field.'.content';
				}
				else if ( $fields_type[$field] == 6 )
				{
					$order .= 'CAST(h.'. $fields_column[$field] .' AS SIGNED)';
				}
				else if ( $fields_type[$field] == 7 )
				{
					$order .= 'CAST(h.'. $fields_column[$field] .' AS DATETIME)';
				}
				else
				{
					$order .= 'h.'.$fields_column[$field];
				}
			}
			$this->_collection->order_default = $order;
			$this->_collection->order_dir_default = '';
		} else {
			$this->_collection->order_default = 'c.ordering';
			$this->_collection->order_dir_default = '';
		}
		$mainframe->setUserState( $option.'.collection.filter_order'.$this->_id, $this->_collection->order_default );
		$mainframe->setUserState( $option.'.collection.filter_order_Dir'.$this->_id, $this->_collection->order_dir_default );
		
		return true;
	}
	
	/**
	 * Tests if collection is checked out
	 *
	 * @access	public
	 * @param	int		$uid	A user id
	 * @return	boolean			True if checked out
	 */
	function isCheckedOut( $uid=0 )
	{
		if ($this->_loadCollection())
		{
			if ($uid) {
				return ($this->_collection->checked_out && $this->_collection->checked_out != $uid);
			} else {
				return $this->_collection->checked_out;
			}
		} elseif ($this->_id < 1) {
			return false;
		} else {
			JError::raiseWarning( 0, 'Unable to Load Data');
			return false;
		}
	}
	
	/**
	 * Tests if collection is published
	 *
	 * @access	public
	 * @return	boolean	True if published
	 */
	function isPublished( )
	{
		global $mainframe;
		
		$now =& JFactory::getDate();
		$nullDate = $this->_db->getNullDate();
		$config =& JFactory::getConfig();
		
		// Test state of the collection
		if ($this->_loadCollection())
		{
			$publish_up =& JFactory::getDate($this->_collection->publish_up);
			$publish_down =& JFactory::getDate($this->_collection->publish_down);
			$publish_up->setOffset($config->getValue('config.offset'));
			$publish_down->setOffset($config->getValue('config.offset'));
			
			if ( $this->_collection->state == 1 )
			{
				if ( $now->toUnix() <= $publish_up->toUnix() )
				{
					return true;
				}
				else if ( ($now->toUnix() <= $publish_down->toUnix() || $this->_collection->publish_down == $nullDate ) )
				{
					return true;
				}
				else if ( $now->toUnix() > $publish_down->toUnix() )
				{
					return false;
				}
			}
			else if ( $this->_collection->state == 0 )
			{
				return false;
			}
			else if ( $this->_collection->state == -1 )
			{
				return false;
			}
		} elseif ($this->_id < 1) {
			return false;
		} else {
			JError::raiseWarning( 0, 'Unable to Load Data');
			return false;
		}
	}
	
	/**
	 * Method to load collection template
	 * Put data in <var>_template</var>
	 *
	 * @access	private
	 * @return	boolean	True on success
	 */
	function _loadTemplate()
	{
		global $mainframe;
		
		if($this->_id == '0')
		{
			return false;
		}
		
		// Load the collection if it doesn't already exist
		if (empty($this->_template))
		{
			$query = 'SELECT *';
			$query .= ' FROM #__collector_templates';
			$query .= ' WHERE client = 1';
			$query .= ' AND collection = '. $this->_id;
			$query .= ' AND home = 1' ;
			$this->_db->setQuery($query);
			$this->_template = $this->_db->loadObject();
			
			if ( empty($this->_template) ) {
				$query = 'SELECT *';
				$query .= ' FROM #__collector_templates';
				$query .= ' WHERE client = 1';
				$query .= ' AND collection = 0';
				$this->_db->setQuery($query);
				$this->_template = $this->_db->loadObject();
				
				if ( empty($this->_template) ) {
					return false;
				}
			}
		}
		return true;
	}
	
	/**
	 * Method to get collection template
	 *
	 * @access	public
	 * @return	object	Template object
	 */
	function &getTemplate()
	{
		// Load the collection data
		if (!$this->_loadTemplate())
		{
			JError::raiseError( 404, JText::_('COM_COLLECTOR_ERROR_LOADING_TEMPLATE') );
		}
		
		return $this->_template;
	}
	
	/**
	 * Method to get number of results
	 * Put data in <var>_total</var>
	 *
	 * @access	public
	 * @return	int		Number of results
	 */
	function getTotal()
	{
		// Load the content if it doesn't already exist
		if (empty($this->_total))
		{
			$query = $this->_buildQuery();
			$this->setBigSelects();
			$this->_total = $this->_getListCount($query);
		}
		return $this->_total;
	}
	
	/**
	 * Method to get pagination
	 * Put data in <var>_pagination</var>
	 * Returns a reference to the global {@link JPagination} object
	 *
	 * @access	public
	 * @return	object JPagination
	 */
	function getPagination()
	{
		// Load the content if it doesn't already exist
		if (empty($this->_pagination))
		{
			jimport('joomla.html.pagination');
			$this->_pagination = new JPagination($this->getTotal(), $this->getState('limitstart'), $this->getState('limit') );
		}
		return $this->_pagination;
	}
	
	/**
	 * Method to load a select box for order
	 *
	 * @access	public
	 * @return	string HTML select order
	 */
	function getOrder()
	{
		global $mainframe, $option;
		
		if(version_compare(JVERSION,'1.6.0','<')){
			//Code pour Joomla! 1.5
			$filter_order		= $mainframe->getUserStateFromRequest( $option.'.collection.filter_order'.$this->_id,		'filter_order',		$this->_collection->order_default,		'string' );
			$filter_order_Dir	= $mainframe->getUserStateFromRequest( $option.'.collection.filter_order_Dir'.$this->_id,	'filter_order_Dir',	$this->_collection->order_dir_default,	'word' );
		}else{
			//Code pour Joomla >= 1.6.0
			$app = JFactory::getApplication();
			$filter_order		= $app->getUserStateFromRequest( $option.'.collection.filter_order'.$this->_id,		'filter_order',		$this->_collection->order_default,		'string' );
			$filter_order_Dir	= $app->getUserStateFromRequest( $option.'.collection.filter_order_Dir'.$this->_id,	'filter_order_Dir',	$this->_collection->order_dir_default,	'word' );
		}
			
			
		$order[] = JHTML::_('select.option',  $this->_collection->order_default, '- '. JText::_( 'COM_COLLECTOR_SELECT_ORDER' ) .' -' );
		
		$i = 0;
		
		foreach ( $this->_collection->fields as $field )
		{
			if ( $field->sort == 1 )
			{
				if ( $field->type == 3 )
				{
					$filter = 'd'.$field->id.'.content';
				}
				else if ( $field->type == 6 )
				{
					$filter = 'CAST(h.'.$field->tablecolumn.' AS SIGNED)';
				}
				else if (( $field->type != 2 ) && ( $field->type != 5 ) && ( $field->type != 8 ))
				{
					$filter = 'h.'.$field->tablecolumn;
				}
				$order[] = JHTML::_('select.option',  $filter, JText::_( $field->field ) );
				$i++;
			}
		}
		
		$list = JHTML::_('select.genericlist',   $order, 'order', 'class="inputbox" size="1" onchange="listOrdering();"', 'value', 'text', $filter_order );
		
		if ( $filter_order_Dir != 'desc' )
		{
			$alt = 'asc';
			$invert = 'desc';
			$img = '/images/sort_asc.png';
		}
		else
		{
			$alt = 'desc';
			$invert = 'asc';
			$img = '/images/sort_desc.png';
		}
		$sort = '<a href="#" onclick="invertDir(\''.$invert.'\');" ><img alt="'.$alt.'" src="'.$img.'" /></a>';
		
		if ( $i > 0 )
		{
			return JText::_( 'COM_COLLECTOR_ORDER_BY' ) . ' ' . $list . ' ' . $sort;
		}
		else
		{
			return '';
		}
	}
	
	/**
	 * Method to force SQL_BIG_SELECTS
	 *
	 * @access	public
	 * @return	void
	 */
	function setBigSelects()
	{
		$db =& JFactory::getDBO();
		
		$query = 'SET SQL_BIG_SELECTS=1';
		$db->setQuery( $query );
		$db->query();
	}
}

}else{
//Code pour Joomla >= 1.6.0
jimport( 'joomla.application.component.modellist' );

require_once(JPATH_ROOT.DS.'administrator'.DS.'components'.DS.'com_collector'.DS.'classes'.DS.'field.php');

/**
 * Collection model
 * @package	Collector
 */
class CollectorModelCollection extends JModelList
{
	/**
	 * Collection Id
	 * @var int
	 */
	var $_collection;
	
	/**
	 * Fields
	 * @var int
	 */
	var $_fields = array();
	
	/**
	 * To know if there is a research
	 * @var boolean
	 */
	var $_search = 0;
	
	/**
	 * Constructor.
	 *
	 * @param	array	An optional associative array of configuration settings.
	 * @see		JController
	 * @since	1.6
	 */
	public function __construct($config = array())
	{
		if (empty($config['filter_fields'])) {
			$config['filter_fields'] = array(
				'id', 'i.id',
				'alias', 'i.alias',
				'collection', 'i.collection',
				'state', 'i.state',
				'ordering', 'i.ordering',
				'hits', 'i.hits',
				'created', 'i.created',
				'created_by', 'i.created_by',
				'checked_out', 'i.checked_out',
				'checked_out_time', 'i.checked_out_time',
				'publish_up', 'i.publish_up',
				'publish_down', 'i.publish_down',
				'access', 'i.access', 'access_level',
			);
		}

		parent::__construct($config);
	}
	
	/**
	 * Method to auto-populate the model state.
	 *
	 * Note. Calling getState in this method will result in recursion.
	 *
	 * return	void
	 * @since	1.6
	 */
	protected function populateState($ordering = null, $direction = null)
	{
		// Initiliase variables.
		$app	= JFactory::getApplication();
		$pk		= JRequest::getInt('id');
		
		$this->setState('collection.id', $pk);

		// Load the parameters. Merge Global and Menu Item params into new object
		$params = $app->getParams();
		$menuParams = new JRegistry;

		if ($menu = $app->getMenu()->getActive()) {
			$menuParams->loadString($menu->params);
		}

		$mergedParams = clone $menuParams;
		$mergedParams->merge($params);

		$this->setState('params', $mergedParams);
		$user		= JFactory::getUser();
		
		// Create a new query object.
		$db		= $this->getDbo();
		$query	= $db->getQuery(true);
		$groups	= implode(',', $user->getAuthorisedViewLevels());

		if ((!$user->authorise('core.edit.state', 'com_collector.collection.'.(int) $pk)) &&  (!$user->authorise('core.edit', 'com_collector.collection.'.(int) $pk))){
			// limit to published for people who can't edit or edit.state.
			$this->setState('filter.published', 1);
			// Filter by start and end dates.
			$nullDate = $db->Quote($db->getNullDate());
			$nowDate = $db->Quote(JFactory::getDate()->toMySQL());

			$query->where('(i.publish_up = ' . $nullDate . ' OR i.publish_up <= ' . $nowDate . ')');
			$query->where('(i.publish_down = ' . $nullDate . ' OR i.publish_down >= ' . $nowDate . ')');
		}
		else {
			$this->setState('filter.published', array(0, 1, 2));
		}

		// process show_noauth parameter
		if (!$params->get('show_noauth')) {
			$this->setState('filter.access', true);
		}
		else {
			$this->setState('filter.access', false);
		}

		// filter
		$itemid = JRequest::getInt('id', 0) . ':' . JRequest::getInt('Itemid', 0);
		$reset = JRequest::getInt('reset', 0);
		if ( $reset == 1 ) {
			$app->setUserState('com_collector.collection.' . $itemid . '.filter_search_all', '');
			$search_all_value = '';
		} else {
			$search_all_value = $app->getUserStateFromRequest('com_collector.collection.'.$itemid.'.filter_search_all', 'filter_search_all', '');
		}
		$this->setState('filter.filter_search_all', $search_all_value);
		
		$orderCol = $app->getUserStateFromRequest('com_collector.collection.' . $itemid . '.filter_order', 'filter_order', 'i.ordering', 'string');
		// if (!in_array($orderCol, $this->filter_fields)) {
			// $orderCol = 'i.ordering';
		// }
		$this->setState('list.ordering', $orderCol);

		$listOrder = $app->getUserStateFromRequest('com_collector.collection.' . $itemid . '.filter_order_Dir',	'filter_order_Dir', '', 'cmd');
		if (!in_array(strtoupper($listOrder), array('ASC', 'DESC', ''))) {
			$listOrder = 'ASC';
		}
		$this->setState('list.direction', $listOrder);

		$this->setState('list.start', JRequest::getVar('limitstart', 0, '', 'int'));

		$limit = $app->getUserStateFromRequest('com_collector.collection.' . $itemid . '.limit', 'limit', $params->get('display_num')?$params->get('display_num'):$app->getCfg('list_limit'));
		
		$this->setState('list.limit', $limit);

		$this->setState('layout', JRequest::getCmd('layout'));
	}

	/**
	 * Method to load default collection Id in <var>_collection</var>
	 *
	 * @access	private
	 */
	public function getCollection()
	{
		$user		= JFactory::getUser();
		
		if ( empty($this->_collection) )
		{
			$collection = $this->getState('collection.id');
			
			if ($collection == '')
			{
				$query = 'SELECT id';
				$query .= ' FROM `#__collector`';
				$query .= ' WHERE home = 1';
				
				$row = $this->_getList( $query );
				
				if ( !$row )
				{
					/**
					 * tag  defaut d'une collection
					 */
					$query = 'UPDATE `#__collector` SET home = 1 LIMIT 1;';
					$this->_db->setQuery( $query );
					$this->_db->query();
					
					$query = 'SELECT id';
					$query .= ' FROM `#__collector`';
					$query .= ' WHERE home = 1';
					
					$row = $this->_getList( $query );
				}
				if ( $row == null )
				{
					return false;
				}
				else
				{
					$collection = $row[0]->id;
				}
			}
			
			// Create a new query object.
			$db =& JFactory::getDBO();
			$query = $db->getQuery(true);
			
			// Select the required fields from the table.
			$query->select('c.*, u.name AS author');
			$query->from('#__collector AS c');
			$query->join('LEFT', '#__users AS u ON u.id = c.created_by');
			// Filter by start and end dates.
			$nullDate = $db->Quote($db->getNullDate());
			$nowDate = $db->Quote(JFactory::getDate()->toMySQL());

			$query->where('((c.created_by = '.$user->id.') OR (c.state = 1 AND ( c.publish_up = '.$nullDate.' OR c.publish_up <= '.$nowDate.') AND ( c.publish_down = '.$nullDate.' OR c.publish_down >= '.$nowDate.' )))');
			
			// Add the filder on ID
			$query->where('c.id = '.$collection);
			
			$db->setQuery( $query );
			
			$this->_collection = $db->loadObject();
			
			if ($this->_collection->custom == 0)
			{
				$query = $db->getQuery(true);
				$query->select('id');
				$query->from('#__collector_fields');
				$query->where('collection = '.$collection);
				$query->where('home = 1');
				
				$db->setQuery( $query );
				
				$this->_collection->custom = $db->loadResult();
			}
			
			$this->_collection->custom = explode('/',$this->_collection->custom);
		}
		
		return $this->_collection;
	}
	
	/**
	 * Method to load fields informations
	 *
	 * @access	private
	 * @return	mixed			Array of fields objects. False if no fields loaded.
	 */
	function getFields()
	{
		if ( empty($this->_fields) )
		{
			if ( $this->_collection->id == '0' )
			{
				return false;
			}
			else
			{
				$collection = $this->_collection->id;
			}
			
			$db		= $this->getDbo();
			$query	= $db->getQuery(true);
			
			$user		=& JFactory::getUser();
			$aid		= (int) $user->get('aid', 0);
			
			$jnow		=& JFactory::getDate();
			$now		= $jnow->toMySQL();
			$nullDate	= $db->getNullDate();
			
			$query->select('f.*, u.name AS author, u.usertype');
			$query->from('#__collector_fields AS f');
			
			// Join over the type.
			$query->select('t.type AS type');
			$query->join('LEFT', '#__collector_fields_type AS t ON t.id = f.type');
			
			$query->join('LEFT', '#__users AS u ON u.id = f.created_by');
			$query->where('collection = ' . $collection);
			
			$query->where('( f.created_by = ' . (int) $user->id . ' OR ( f.state = 1 AND ( f.publish_up = '.$db->Quote($nullDate).' OR f.publish_up <= '.$db->Quote($now).' ) AND ( f.publish_down = '.$db->Quote($nullDate).' OR f.publish_down >= '.$db->Quote($now).' ) ) )');
			
			// Filter by access level.
			if ($access = $this->getState('filter.access')) {
				$user	= JFactory::getUser();
				$groups	= implode(',', $user->getAuthorisedViewLevels());
				$query->where('f.access IN ('.$groups.')');
			}
			$query->order('ordering');
			
			$db->setQuery($query);
			$fields = $db->loadObjectList();
			
			if ( ! $fields ) {
				return false;
			}
			
			foreach ($fields as $field)
			{
				$registry = new JRegistry;
				$registry->loadString($field->attribs);
				$field->attribs = $registry->toArray();
				$this->_fields[] =& CollectorField::getInstance( $this->_collection->id, $field );
			}
		}
		return $this->_fields;
	}
	
	/**
	 * Build an SQL query to load the list data.
	 *
	 * @return	JDatabaseQuery
	 */
	protected function getListQuery()
	{
		// Get collection
		$app		= JFactory::getApplication();
		$collection	= $this->getCollection();
		$fields		= $this->getFields();
		$params		= $app->getParams();
		
		$reset = JRequest::getInt('reset', 0);
		
		$user	= JFactory::getUser();
		$groups	= implode(',', $user->getAuthorisedViewLevels());
		
		// Create a new query object.
		$db			= $this->getDbo();
		$nullDate	= $db->getNullDate();
		$jnow		=& JFactory::getDate();
		$now		= $jnow->toMySQL();
		$query		= $db->getQuery(true);
		
		// Select the required fields from the table.
		$query->select(
			$this->getState(
				'list.select',
				'i.id, i.alias, i.collection, i.ordering, i.state, i.hits, i.created, i.created_by, i.created_by_alias, i.modified, i.modified_by, ' .
				'i.access, i.publish_up, i.publish_down, i.checked_out, i.checked_out_time, ' .
				// use created if modified is 0
				'CASE WHEN i.modified = 0 THEN i.created ELSE i.modified END as modified, ' .
					'i.modified_by, uam.name as modified_by_name'
			)
		);
		$query->from('#__collector_items AS i');
		
		// Join over the values.
		$query->join('LEFT', '#__collector_items_history_'.$collection->id.' AS h ON h.item = i.id');
		foreach($fields as $field)
		{
			$field->setQuery($query);
		}
		
		// Join over the users for the author and modified_by names.
		$query->select("CASE WHEN i.created_by_alias > ' ' THEN i.created_by_alias ELSE ua.name END AS author");
		$query->join('LEFT', '#__users AS ua ON ua.id = i.created_by');
		$query->join('LEFT', '#__users AS uam ON uam.id = i.modified_by');
		
		$query->where('h.state = 1');
		$query->where('i.collection = '.$collection->id);
		$query->where('i.access IN ('.$groups.')');
		$query->where('((i.created_by = '.$user->id.') OR (i.state = 1 AND ( i.publish_up = '.$db->Quote($nullDate).' OR i.publish_up <= '.$db->Quote($now).') AND ( i.publish_down = '.$db->Quote($nullDate).' OR i.publish_down >= '.$db->Quote($now).' )))');
		
		$whereSearch = array();
		
		$itemid = JRequest::getInt('id', 0) . ':' . JRequest::getInt('Itemid', 0);
		
		$filter = $params->get('filter');
		
		foreach($fields as $field)
		{
			if ( $field->_field->filter == 1 )
			{
				$nameFilterCollection = 'filterfield_'.$field->_field->tablecolumn;
				if (isset($filter[$nameFilterCollection])) {
					$valueFilterMenu = $filter[$nameFilterCollection];
				} else {
					$valueFilterMenu = '';
				}

				if ( $reset == 1 ) {
					$app->setUserState('com_collector.collection.' . $itemid . '.'.$nameFilterCollection, $valueFilterMenu);
					$filtervalue = $valueFilterMenu;
				} else {
					$filtervalue = $app->getUserStateFromRequest('com_collector.collection.' . $itemid . '.'.$nameFilterCollection, $nameFilterCollection, $valueFilterMenu);
				}
				$this->setState('filter.'.$nameFilterCollection, $filtervalue);
				
				if ($field->setFilterWhereClause($query,$filtervalue))
				{
					$this->_search = 1;
				}
			}
			
			$search_all_value = $this->getState('filter.filter_search_all');
			if ( $search_all_value != '' )
			{
				$this->_search = 1;
				$whereSearch[] = $field->getSearchWhereClause($query,$search_all_value);
			}
		}
		
		if (count($whereSearch) > 0 ) {
			$query->where('('.implode(' OR ', $whereSearch).')');
		}
		
		// Add the list ordering clause.
		$query->order($this->_buildCollectionOrderBy());
		
		return $query;
	}
	
	/**
	 * Method to build the ORDER BY clause of the query to select items
	 *
	 * @access	private
	 * @return	string	ORDER BY clause
	 */
	function _buildCollectionOrderBy()
	{
		$app = JFactory::getApplication();
		
		$itemid = JRequest::getInt('id', 0) . ':' . JRequest::getInt('Itemid', 0);
		$reset = JRequest::getInt('reset', 0);
		
		$fields_order = array();
		
		$order = '';
		
		$collection	= $this->getCollection();
		$fields		= $this->getFields();
		$params		= $app->getParams();
		
		$special_order		= $params->get('order');
		if ( $special_order == 'ordering' ) {
			$order_default = 'i.ordering';
			$order_dir_default = '';
		} else if ( $special_order == 'created' ) {
			$order_default = 'i.created';
			$order_dir_default = '';
		} else if ( $special_order == 'rcreated' ) {
			$order_default = 'i.created';
			$order_dir_default = 'desc';
		} else if ( $special_order == 'modified' ) {
			$order_default = 'h.modified';
			$order_dir_default = 'desc';
		} else if ( $special_order == 'rmodified' ) {
			$order_default = 'h.modified';
			$order_dir_default = '';
		} else if ( $special_order == 'default' ) {
			foreach ($fields as $field)
			{
				$fields_order[$field->_field->id] = $field->getOrderBy();
			}
			foreach ( $collection->custom as $field )
			{
				if ( $order != '' )
				{
					$order .= ' , ';
				}
				$order .= $fields_order[$field];
			}
			$order_default = $order;
			$order_dir_default = '';
		} else {
			$order_default = 'i.ordering';
			$order_dir_default = '';
		}
		
		if ( $reset ) {
			$app->setUserState('com_collector.collection.' . $itemid . '.filter_order', $order_default);
			$this->setState('list.ordering', $order_default);
			$app->setUserState('com_collector.collection.' . $itemid . '.filter_order_Dir', $order_dir_default);
			$this->setState('list.direction', $order_dir_default);
		}
		
		$orderby = $this->getState('list.ordering').' '.$this->getState('list.direction');
		
		return $orderby;
	}
	
	/**
	 * Tests if collection is searched
	 *
	 * @access	public
	 * @return	boolean	True if searched
	 */
	function getSearched()
	{
		if ( $this->_search == 1 )
		{
			return true;
		}
		
		return false;
	}
}
}
