<?php
/**
 * Joomla! 1.5 component Collector
 *
 * @version 	$Id: filelist.php 134 2012-04-05 20:45:38Z steevo $
 * @author 		Philippe Ousset steevo@steevo.fr
 * @package 	Collector
 * @license 	GNU/GPL
 *
 * Collector is a Multi Purpose Listing Tool.
 * Originaly developped to list Collections
 * it can be used for several purpose.
 */

// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die( 'Restricted access' );

jimport('joomla.application.component.model');
jimport('joomla.filesystem.folder');
jimport('joomla.filesystem.file');

if(version_compare(JVERSION,'1.6.0','<')){
//Code pour Joomla! 1.5

/**
 * Filelist model
 * @package		Collector
 */
class CollectorModelFilelist extends JModel
{
	/**
	 * The relative Path from site root to the current folder
	 * @var string 
	 */
	var $_folder;

	/**
	 * The type of files
	 * @var int 
	 */
	var $_type;

	/**
	 * The path to the current folder
	 * @var array 
	 */
	var $_path;

	/**
	 * Filelist model constructor
	 * @access	public
	 */
	function __construct()
	{
		parent::__construct();
		
		global $mainframe, $option;
		
        // Get folder request variables
		if(version_compare(JVERSION,'1.6.0','<')){
			//Code pour Joomla! 1.5
			$this->_folder = $mainframe->getUserStateFromRequest($option.'.filemanager.folder', 'folder', '', 'text');
		}else{
			//Code pour Joomla >= 1.6.0
			$app = JFactory::getApplication();
			$this->_folder = $app->getUserStateFromRequest($option.'.filemanager.folder', 'folder', '', 'text');
		}
		
		$this->_type = JRequest::getInt('type', 0);
		
		$this->_path = array();
		$tmp = new JObject();
		$tmp->name = JText::_( 'COM_COLLECTOR_ROOT' );
		$tmp->path = '';
		$this->_path[] = $tmp;
		
		if ( $this->_folder != '' )
		{
			$this->cleanPath();
		}
	}
	
	/**
	 * Retrieve the current folder
	 *
	 * @access	public
	 * @return	string	Relative Path from site root
	 */
	function getFolder()
	{
		return $this->_folder;
	}
	
	/**
	 * Retrieve the type of files
	 *
	 * @access	public
	 * @return	int		Id of type
	 */
	function getType()
	{
		return $this->_type;
	}
	
	/**
	 * Retrieves list of folders and files from a folder
	 *
	 * @access	public
	 * @param	string	$baseFolder		The path of the folder to read
	 * @return	array					Array of objects containing the data from the filesystem
	 */
	function getList($baseFolder = null)
	{
		$baseSite = str_replace(DS, '/', JPATH_ROOT.DS);
		
		//Si aucun dossier specifie on va a la racine du site
		if ( $baseFolder == null )
		{
			if ( $this->_folder == '' )
			{
				$baseFolder = JPATH_ROOT.DS;
			}
			else
			{
				$baseFolder = JPATH_ROOT.DS.$this->_folder;
				
				$precFolder = implode ( '/' , explode( '/', $baseFolder, -1 ) );
				
				$prec = new JObject();
				$prec->name = JText::_('COM_COLLECTOR_BACK_DIRECTORY');
				$prec->type = '';
				$prec->ico = 'administrator/components/com_collector/assets/images/folder_back.png';
				$prec->path = str_replace(DS, '/', JPath::clean($precFolder));
				if ( $prec->path.'/' == $baseSite )
				{
					$prec->path_relative = '';
				}
				else
				{
					$prec->path_relative = str_replace($baseSite, '', $prec->path);
				}
				//$count = MediaHelper::countFiles($prec->path);
				//$prec->files = $count[0];
				//$prec->folders = $count[1];
				
				$folders[] = $prec;
			}
		}
		else
		{
			$baseFolder = JPATH_ROOT.DS.$baseFolder;
		}
		
		$extensions = $this->getExtensions();
		
		if ( !(JFolder::exists($baseFolder)) )
		{
			JFolder::create($baseFolder);
		}
		$foldersList = JFolder::folders($baseFolder);
		$filesList = JFolder::files($baseFolder);
		
		$files = array();
		
		if ( $filesList != null )
		{
			foreach ( $filesList as $file )
			{
				$ext = strtolower(JFile::getExt($file));
				
				if ( array_key_exists( $ext, $extensions ) )
				{
					$tmp = new JObject();
					$tmp->name = $file;
					$tmp->path = str_replace(DS, '/', JPath::clean($baseFolder.DS.$file));
					$tmp->path_relative = str_replace($baseSite, '', $tmp->path);
					$tmp->size = $this->get_file_resize($tmp->path);
					$tmp->type = $extensions[$ext]->type;
					$tmp->text = JText::_('COM_COLLECTOR_'.$extensions[$ext]->text);
					$tmp->ico = 'administrator/components/com_collector/assets/images/'.$extensions[$ext]->ico;
					$files[] = $tmp;
				}
			}
			
		}
		
		if ( $foldersList != null )
		{
			foreach ( $foldersList as $folder )
			{
				$tmp = new JObject();
				$tmp->name = basename($folder);
				$tmp->text = JText::_('COM_COLLECTOR_DIRECTORY');
				$tmp->ico = 'administrator/components/com_collector/assets/images/folder.png';
				$tmp->path = str_replace(DS, '/', JPath::clean($baseFolder.DS.$folder));
				$tmp->path_relative = str_replace($baseSite, '', $tmp->path);
				//$count = MediaHelper::countFiles($tmp->path);
				//$tmp->files = $count[0];
				//$tmp->folders = $count[1];
				
				$folders[] = $tmp;
			}
		}
		
		$this->list = array('folders' => $folders, 'files' => $files);
		
		return $this->list;
	}
	
	/**
	 * Retrieve extensions available
	 *
	 * @access	public
	 * @return	array	Array of objects with extensions available indexed by extension
	 */
	function getExtensions()
	{
		$query = 'SELECT e.id, t.text, e.type, e.ext, e.ico, e.state';
		$query .= ' FROM `#__collector_files_ext` AS e';
		$query .= ' LEFT JOIN `#__collector_files_type` AS t ON e.type = t.id';
		$query .= ' WHERE e.state = "1"';
		
		if ( $this->_type == 1 )
		{
			$query .= ' AND e.type = "1"';
		}
		
		$db =& $this->getDBO();
		
		$db->setQuery($query);
		$db->query();
		
		$this->_ext = $db->loadObjectList ( 'ext' );
		
		return $this->_ext;
	}
	
	/**
	 * Retrieve size of an image from filesystem
	 *
	 * @access	public
	 * @param	string	$filename	The path of the file to analyse
	 * @return	array				Size of the image
	 */
	function get_file_resize($filename)
	{
		$maxwidth = 184;
		$maxheight = 245;
		
		$taille=getimagesize($filename);
		
		$width=$taille[0];
		$height=$taille[1];
		
		if ( ( $height < $maxheight ) && ( $width < $maxwidth ) )
		{
			$widthresized = $width;
			$heightresized = $height;
		}
		else
		{
			if ( ($height/$maxheight) <= ($width/$maxwidth) )
			{
				$widthresized = $maxwidth;
				$heightresized = ($maxwidth*$height)/$width;
			}
			else
			{
				$widthresized = ($maxheight*$width)/$height;
				$heightresized = $maxheight;
			}
		}
		
		$margin = ($maxheight-$heightresized)/2;
		
		return array($widthresized,$heightresized,$margin);
	}
	
	/**
	 * Check folder from request
	 * Clean <var>_folder</var>
	 *
	 * @access	public
	 * @return	void
	 */
	function cleanPath()
	{
		$this->_folder = str_replace('\\','/',$this->_folder);
		$path = explode('/',$this->_folder);
		
		$i = 0;
		
		foreach ( $path as $folder )
		{
			if ( $folder == '..' )
			{
				$i = $i - 1;
				if ( $i < 0 )
				{
					$cleanPath = array();
					$this->_path = array();
					$tmp = new JObject();
					$tmp->name = JText::_( 'COM_COLLECTOR_ROOT' );
					$tmp->path = '';
					$this->_path[] = $tmp;
					break;
				}
				else
				{
					unset($cleanPath[$i]);
					unset($this->_path[$i]);
				}
			}
			else
			{
				if ( $folder != '.' )
				{
					$cleanPath[] = $folder;
					$tmp = new JObject();
					$tmp->name = $folder;
					$tmp->path = implode('/', $cleanPath);
					$this->_path[] = $tmp;
					$i = $i + 1;
				}
			}
		}
		
		$this->_folder = implode('/', $cleanPath);
	}
	
	/**
	 * Construct Path links for navigation in the filesystem
	 *
	 * @access	public
	 * @return	string	Navigation html
	 */
	function get_path_navigation()
	{
		$navigation = '';
		
		foreach ($this->_path as $folder)
		{
			$navigation .= '<a href="' . JRoute::_( 'index.php?option=com_collector&amp;view=filelist&amp;tmpl=component&amp;folder='.$folder->path.'&amp;type='.$this->_type ) . '" >' . $folder->name . '</a>/';
		}
		
		return $navigation;
	}
	
	/**
	 * Method to create folder
	 *
	 * @access	public
	 * @return	string	Message to display
	 */
	function createFolder()
	{
		$folder = JRequest::getVar('foldername', '', '', 'text');
		
		if (strlen($folder) > 0) {
			$path = JPath::clean(JPATH_ROOT.DS.$this->_folder.DS.$folder);
			if (!is_dir($path))
			{
				if (!is_file($path))
				{
					JFolder::create($path);
					JFile::write($path.DS."index.html", "<html><body bgcolor=\"#FFFFFF\"></body></html>");
					$msg[0] = '"' . $folder . '" ' . JText::_( 'COM_COLLECTOR_FOLDER_CREATED' );
					$msg[1] = 'message';
				}
				else
				{
					$msg[0] = JText::_( 'COM_COLLECTOR_EXIST_FILE_SAME' );
					$msg[1] = 'error';
				}
			}
			else
			{
				$msg[0] = JText::_( 'COM_COLLECTOR_EXIST_FOLDER_SAME' );
				$msg[1] = 'error';
			}
			
		}
		else
		{
			$msg[0] = JText::_( 'COM_COLLECTOR_NO_FOLDER_NAME' );
			$msg[1] = 'error';
		}
		
		return $msg;
	}
}

}else{

//Code pour Joomla >= 1.6.0
jimport('joomla.application.component.modellist');

/**
 * Filelist model
 * @package	Collector
 */
class CollectorModelFilelist extends JModel
{
	/**
	 * Folder
	 * @var string
	 */
	var $_folder;
	
	/**
	 * Files Type
	 * @var int
	 */
	var $_type;
	
	/**
	 * Path to folder
	 * @var array
	 */
	var $_path;
	
	/**
	 * Extensions datas
	 * @var array
	 */
	var $_ext;
	
	/**
	 * Filelist model constructor
	 *
	 * @access	public
	 */
	function __construct()
	{
		parent::__construct();
		
        // Get folder request variables
		$app = JFactory::getApplication();
		$this->_folder = $app->getUserStateFromRequest('collector.filemanager.folder', 'folder', '', 'text');
		
		$this->_type = JRequest::getInt('type', 0);
		
		$this->_path = array();
		$tmp = new JObject();
		$tmp->name = JText::_( 'COM_COLLECTOR_ROOT' );
		$tmp->path = '';
		$this->_path[] = $tmp;
		
		if ( $this->_folder != '' )
		{
			$this->cleanPath();
		}
	}
	
	/**
	 * Method to get current folder
	 *
	 * @access	public
	 * @return	string	Relative Path from site root
	 */
	function getFolder()
	{
		return $this->_folder;
	}
	
	/**
	 * Method to get type of files
	 *
	 * @access	public
	 * @return	int	Type of files to display
	 */
	function getType()
	{
		return $this->_type;
	}
	
	/**
	 * Retrieves list of folders and files from a folder
	 *
	 * @access	public
	 * @param	string	$baseFolder	Relative Path from site root
	 * @return	array				Array of objects containing the data from the filesystem
	 */
	function getList($baseFolder = null)
	{
		$baseSite = str_replace(DS, '/', JPATH_ROOT.DS);
		
		//Si aucun dossier specifie on va a la racine du site
		if ( $baseFolder == null )
		{
			if ( $this->_folder == '' )
			{
				$baseFolder = JPATH_ROOT.DS;
			}
			else
			{
				$baseFolder = JPATH_ROOT.DS.$this->_folder;
				
				$precFolder = implode ( '/' , explode( '/', $baseFolder, -1 ) );
				
				$prec = new JObject();
				$prec->name = JText::_('COM_COLLECTOR_BACK_DIRECTORY');
				$prec->type = '';
				$prec->ico = 'administrator/components/com_collector/assets/images/folder_back.png';
				$prec->path = str_replace(DS, '/', JPath::clean($precFolder));
				if ( $prec->path.'/' == $baseSite )
				{
					$prec->path_relative = '';
				}
				else
				{
					$prec->path_relative = str_replace($baseSite, '', $prec->path);
				}
				//$count = MediaHelper::countFiles($prec->path);
				//$prec->files = $count[0];
				//$prec->folders = $count[1];
				
				$folders[] = $prec;
			}
		}
		else
		{
			$baseFolder = JPATH_ROOT.DS.$baseFolder;
		}
		
		$extensions = $this->getExtensions();
		
		if ( !(JFolder::exists($baseFolder)) )
		{
			JFolder::create($baseFolder);
		}
		$foldersList = JFolder::folders($baseFolder);
		$filesList = JFolder::files($baseFolder);
		
		$files = array();
		
		if ( $filesList != null )
		{
			foreach ( $filesList as $file )
			{
				$ext = strtolower(JFile::getExt($file));
				
				if ( array_key_exists( $ext, $extensions ) )
				{
					$tmp = new JObject();
					$tmp->name = $file;
					$tmp->path = str_replace(DS, '/', JPath::clean($baseFolder.DS.$file));
					$tmp->path_relative = str_replace($baseSite, '', $tmp->path);
					$tmp->size = $this->get_file_resize($tmp->path);
					$tmp->type = $extensions[$ext]->type;
					$tmp->text = JText::_('COM_COLLECTOR_'.$extensions[$ext]->text);
					$tmp->ico = 'administrator/components/com_collector/assets/images/'.$extensions[$ext]->ico;
					$files[] = $tmp;
				}
			}
			
		}
		
		if ( $foldersList != null )
		{
			foreach ( $foldersList as $folder )
			{
				$tmp = new JObject();
				$tmp->name = basename($folder);
				$tmp->text = JText::_('COM_COLLECTOR_DIRECTORY');
				$tmp->ico = 'administrator/components/com_collector/assets/images/folder.png';
				$tmp->path = str_replace(DS, '/', JPath::clean($baseFolder.DS.$folder));
				$tmp->path_relative = str_replace($baseSite, '', $tmp->path);
				//$count = MediaHelper::countFiles($tmp->path);
				//$tmp->files = $count[0];
				//$tmp->folders = $count[1];
				
				$folders[] = $tmp;
			}
		}
		
		$this->list = array('folders' => $folders, 'files' => $files);
		
		return $this->list;
	}
	
	/**
	 * Retrieve lists of extensions managed by filemanager
	 *
	 * @access	public
	 * @return	array	Array of objects containing the data from the filesystem
	 */
	function getExtensions()
	{
		$query = 'SELECT e.id, t.text, e.type, e.ext, e.ico, e.state';
		$query .= ' FROM `#__collector_files_ext` AS e';
		$query .= ' LEFT JOIN `#__collector_files_type` AS t ON e.type = t.id';
		$query .= ' WHERE e.state = "1"';
		
		if ( $this->_type == 1 )
		{
			$query .= ' AND e.type = "1"';
		}
		
		$db =& $this->getDBO();
		
		$db->setQuery($query);
		$db->query();
		
		$this->_ext = $db->loadObjectList ( 'ext' );
		
		return $this->_ext;
	}
	
	/**
	 * Retrieve size file from filesystem
	 *
	 * @access	public
	 * @param	string	$filename	Relative Path from site root
	 * @return	array				[new width,new height, margin]
	 */
	function get_file_resize($filename)
	{
		$maxwidth = 184;
		$maxheight = 245;
		
		$taille=getimagesize($filename);
		
		$width=$taille[0];
		$height=$taille[1];
		
		if ( ( $height < $maxheight ) && ( $width < $maxwidth ) )
		{
			$widthresized = $width;
			$heightresized = $height;
		}
		else
		{
			if ( ($height/$maxheight) <= ($width/$maxwidth) )
			{
				$widthresized = $maxwidth;
				$heightresized = ($maxwidth*$height)/$width;
			}
			else
			{
				$widthresized = ($maxheight*$width)/$height;
				$heightresized = $maxheight;
			}
		}
		
		$margin = ($maxheight-$heightresized)/2;
		
		return array($widthresized,$heightresized,$margin);
	}
	
	/**
	 * Check folder from request
	 * Clean <var>_folder</var>
	 *
	 * @access	public
	 * @return	void
	 */
	function cleanPath()
	{
		$this->_folder = str_replace('\\','/',$this->_folder);
		$path = explode('/',$this->_folder);
		
		$i = 0;
		
		foreach ( $path as $folder )
		{
			if ( $folder == '..' )
			{
				$i = $i - 1;
				if ( $i < 0 )
				{
					$cleanPath = array();
					$this->_path = array();
					$tmp = new JObject();
					$tmp->name = JText::_( 'COM_COLLECTOR_ROOT' );
					$tmp->path = '';
					$this->_path[] = $tmp;
					break;
				}
				else
				{
					unset($cleanPath[$i]);
					unset($this->_path[$i]);
				}
			}
			else
			{
				if ( $folder != '.' )
				{
					$cleanPath[] = $folder;
					$tmp = new JObject();
					$tmp->name = $folder;
					$tmp->path = implode('/', $cleanPath);
					$this->_path[] = $tmp;
					$i = $i + 1;
				}
			}
		}
		
		$this->_folder = implode('/', $cleanPath);
	}
	
	/**
	 * Construct Path links for navigation in the filesystem
	 *
	 * @access	public
	 * @return 	string 	Navigation html
	 */
	function get_path_navigation()
	{
		$navigation = '';
		
		foreach ($this->_path as $folder)
		{
			$navigation .= '<a href="' . JRoute::_( 'index.php?option=com_collector&amp;view=filelist&amp;tmpl=component&amp;folder='.$folder->path.'&amp;type='.$this->_type ) . '" >' . $folder->name . '</a>/';
		}
		
		return $navigation;
	}
	
	/**
	 * Method to create folder
	 *
	 * @access	public
	 * @return	string	Message to display
	 */
	function createFolder()
	{
		$folder = JRequest::getVar('foldername', '', '', 'text');
		
		if (strlen($folder) > 0) {
			$path = JPath::clean(JPATH_ROOT.DS.$this->_folder.DS.$folder);
			if (!is_dir($path))
			{
				if (!is_file($path))
				{
					JFolder::create($path);
					JFile::write($path.DS."index.html", "<html><body bgcolor=\"#FFFFFF\"></body></html>");
					$msg[0] = '"' . $folder . '" ' . JText::_( 'COM_COLLECTOR_FOLDER_CREATED' );
					$msg[1] = 'message';
				}
				else
				{
					$msg[0] = JText::_( 'COM_COLLECTOR_EXIST_FILE_SAME' );
					$msg[1] = 'error';
				}
			}
			else
			{
				$msg[0] = JText::_( 'COM_COLLECTOR_EXIST_FOLDER_SAME' );
				$msg[1] = 'error';
			}
			
		}
		else
		{
			$msg[0] = JText::_( 'COM_COLLECTOR_NO_FOLDER_NAME' );
			$msg[1] = 'error';
		}
		
		return $msg;
	}
}

}